@@ -1,5 +1,5 @@
 /*
-  Copyright(C) 2009-2016 Brazil
+  Copyright(C) 2009-2017 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,7 @@ typedef unsigned int grn_expr_flags;
 #define GRN_EXPR_ALLOW_COLUMN          (0x04)
 #define GRN_EXPR_ALLOW_UPDATE          (0x08)
 #define GRN_EXPR_ALLOW_LEADING_NOT     (0x10)
+#define GRN_EXPR_QUERY_NO_SYNTAX_ERROR (0x80)
 
 GRN_API grn_obj *grn_expr_create(grn_ctx *ctx, const char *name, unsigned int name_size);
 GRN_API grn_rc grn_expr_close(grn_ctx *ctx, grn_obj *expr);
@@ -53,6 +54,11 @@ GRN_API grn_obj *grn_expr_append_const_str(grn_ctx *ctx, grn_obj *expr,
                                            grn_operator op, int nargs);
 GRN_API grn_obj *grn_expr_append_const_int(grn_ctx *ctx, grn_obj *expr, int i,
                                            grn_operator op, int nargs);
+GRN_API grn_obj *grn_expr_append_const_bool(grn_ctx *ctx,
+                                            grn_obj *expr,
+                                            grn_bool value,
+                                            grn_operator op,
+                                            int nargs);
 GRN_API grn_rc grn_expr_append_op(grn_ctx *ctx, grn_obj *expr, grn_operator op, int nargs);
 
 GRN_API grn_rc grn_expr_get_keywords(grn_ctx *ctx, grn_obj *expr, grn_obj *keywords);
@@ -70,6 +76,13 @@ GRN_API grn_rc grn_expr_syntax_expand_query(grn_ctx *ctx,
                                             grn_expr_flags flags,
                                             grn_obj *expander,
                                             grn_obj *expanded_query);
+GRN_API grn_rc grn_expr_syntax_expand_query_by_table(grn_ctx *ctx,
+                                                     const char *query,
+                                                     int query_size,
+                                                     grn_expr_flags flags,
+                                                     grn_obj *term_column,
+                                                     grn_obj *expanded_term_column,
+                                                     grn_obj *expanded_query);
 
 GRN_API grn_rc grn_expr_compile(grn_ctx *ctx, grn_obj *expr);
 GRN_API grn_obj *grn_expr_rewrite(grn_ctx *ctx, grn_obj *expr);
