@@ -22,6 +22,7 @@
 #include "grn_request_timer.h"
 #include "grn_tokenizers.h"
 #include "grn_ctx_impl.h"
+#include "grn_encoding.h"
 #include "grn_ii.h"
 #include "grn_pat.h"
 #include "grn_index_column.h"
@@ -39,10 +40,6 @@
 #include <stdarg.h>
 #include <time.h>
 
-#ifdef GRN_WITH_ONIGMO
-# define GRN_SUPPORT_REGEXP
-#endif /* GRN_WITH_ONIGMO */
-
 #ifdef GRN_SUPPORT_REGEXP
 # include <onigmo.h>
 #endif /* GRN_SUPPORT_REGEXP */
@@ -212,55 +209,43 @@ grn_ctx_impl_init(grn_ctx *ctx)
   ctx->impl->encoding = ctx->encoding;
   ctx->impl->lifoseg = -1;
   ctx->impl->currseg = -1;
+
+  ctx->impl->values = NULL;
+  ctx->impl->temporary_columns = NULL;
+  ctx->impl->temporary_options = NULL;
+  ctx->impl->ios = NULL;
+  ctx->impl->expr_vars = NULL;
+  ctx->impl->stack = NULL;
+
   CRITICAL_SECTION_INIT(ctx->impl->lock);
   if (!(ctx->impl->values = grn_array_create(ctx, NULL, sizeof(grn_db_obj *),
                                              GRN_ARRAY_TINY))) {
-    CRITICAL_SECTION_FIN(ctx->impl->lock);
-    grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
-    ctx->impl = NULL;
-    return ctx->rc;
+    goto exit;
   }
   if (!(ctx->impl->temporary_columns = grn_pat_create(ctx, NULL,
                                                       GRN_TABLE_MAX_KEY_SIZE,
                                                       sizeof(grn_obj *),
                                                       0))) {
-    grn_array_close(ctx, ctx->impl->values);
-    CRITICAL_SECTION_FIN(ctx->impl->lock);
-    grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
-    ctx->impl = NULL;
-    return ctx->rc;
+    goto exit;
+  }
+  ctx->impl->temporary_options =
+    grn_options_create(ctx, NULL, "[ctx][impl][create]");
+  if (!ctx->impl->temporary_options) {
+    goto exit;
   }
   if (!(ctx->impl->ios = grn_hash_create(ctx, NULL, GRN_TABLE_MAX_KEY_SIZE,
                                          sizeof(grn_io *),
                                          GRN_OBJ_KEY_VAR_SIZE|GRN_HASH_TINY))) {
-    grn_array_close(ctx, ctx->impl->values);
-    grn_pat_close(ctx, ctx->impl->temporary_columns);
-    CRITICAL_SECTION_FIN(ctx->impl->lock);
-    grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
-    ctx->impl = NULL;
-    return ctx->rc;
+    goto exit;
   }
   ctx->impl->db = NULL;
 
   if (!(ctx->impl->expr_vars = grn_hash_create(ctx, NULL, sizeof(grn_id),
                                                sizeof(grn_obj *), 0))) {
-    grn_array_close(ctx, ctx->impl->values);
-    grn_pat_close(ctx, ctx->impl->temporary_columns);
-    CRITICAL_SECTION_FIN(ctx->impl->lock);
-    grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
-    grn_hash_close(ctx, ctx->impl->ios);
-    ctx->impl = NULL;
-    return ctx->rc;
+    goto exit;
   }
   if (!(ctx->impl->stack = GRN_MALLOCN(grn_obj *, GRN_STACK_SIZE))) {
-    grn_array_close(ctx, ctx->impl->values);
-    grn_pat_close(ctx, ctx->impl->temporary_columns);
-    CRITICAL_SECTION_FIN(ctx->impl->lock);
-    grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
-    grn_hash_close(ctx, ctx->impl->ios);
-    grn_hash_close(ctx, ctx->impl->expr_vars);
-    ctx->impl = NULL;
-    return ctx->rc;
+    goto exit;
   }
   ctx->impl->stack_curr = 0;
   ctx->impl->stack_size = GRN_STACK_SIZE;
@@ -316,6 +301,37 @@ grn_ctx_impl_init(grn_ctx *ctx)
   GRN_TEXT_INIT(&(ctx->impl->temporary_open_spaces.stack), 0);
   ctx->impl->temporary_open_spaces.current = NULL;
 
+exit :
+  if (ctx->rc != GRN_SUCCESS) {
+    GRN_OBJ_FIN(ctx, &(ctx->impl->temporary_open_spaces.stack));
+    GRN_OBJ_FIN(ctx, &ctx->impl->query_log_buf);
+    grn_obj_close(ctx, ctx->impl->output.buf);
+    GRN_OBJ_FIN(ctx, &ctx->impl->output.levels);
+    GRN_OBJ_FIN(ctx, &ctx->impl->output.names);
+    GRN_OBJ_FIN(ctx, &ctx->impl->current_request_id);
+    if (ctx->impl->stack) {
+      GRN_FREE(ctx->impl->stack);
+    }
+    if (ctx->impl->expr_vars) {
+      grn_hash_close(ctx, ctx->impl->expr_vars);
+    }
+    if (ctx->impl->ios) {
+      grn_hash_close(ctx, ctx->impl->ios);
+    }
+    if (ctx->impl->temporary_options) {
+      grn_options_close(ctx, ctx->impl->temporary_options);
+    }
+    if (ctx->impl->temporary_columns) {
+      grn_pat_close(ctx, ctx->impl->temporary_columns);
+    }
+    if (ctx->impl->values) {
+      grn_array_close(ctx, ctx->impl->values);
+    }
+    CRITICAL_SECTION_FIN(ctx->impl->lock);
+    grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
+    ctx->impl = NULL;
+  }
+
   return ctx->rc;
 }
 
@@ -364,6 +380,108 @@ grn_ctx_impl_set_current_error_message(grn_ctx *ctx)
   grn_strcpy(ctx->impl->previous_errbuf, GRN_CTX_MSGSIZE, ctx->errbuf);
 }
 
+static grn_rc
+grn_ctx_impl_fin(grn_ctx *ctx)
+{
+  grn_rc rc;
+
+  grn_ctx_impl_clear_n_same_error_messagges(ctx);
+  if (ctx->impl->finalizer) {
+    ctx->impl->finalizer(ctx, 0, NULL, &(ctx->user_data));
+  }
+  {
+    grn_obj *stack;
+    grn_obj *spaces;
+    unsigned int i, n_spaces;
+
+    stack = &(ctx->impl->temporary_open_spaces.stack);
+    spaces = (grn_obj *)GRN_BULK_HEAD(stack);
+    n_spaces = GRN_BULK_VSIZE(stack) / sizeof(grn_obj);
+    for (i = 0; i < n_spaces; i++) {
+      grn_obj *space = spaces + (n_spaces - i - 1);
+      GRN_OBJ_FIN(ctx, space);
+    }
+    GRN_OBJ_FIN(ctx, stack);
+  }
+  grn_ctx_impl_mrb_fin(ctx);
+  grn_ctx_loader_clear(ctx);
+  if (ctx->impl->parser) {
+    grn_expr_parser_close(ctx);
+  }
+  GRN_OBJ_FIN(ctx, &ctx->impl->current_request_id);
+  if (ctx->impl->values) {
+#ifndef USE_MEMORY_DEBUG
+    grn_db_obj *o;
+    GRN_ARRAY_EACH(ctx, ctx->impl->values, 0, 0, id, &o, {
+      grn_obj_close(ctx, *((grn_obj **)o));
+    });
+#endif
+    grn_array_close(ctx, ctx->impl->values);
+  }
+  if (ctx->impl->temporary_columns) {
+#ifndef USE_MEMORY_DEBUG
+    grn_obj *value;
+    GRN_PAT_EACH(ctx, ctx->impl->temporary_columns, id, NULL, NULL, &value, {
+      grn_obj_close(ctx, *((grn_obj **)value));
+    });
+#endif
+    grn_pat_close(ctx, ctx->impl->temporary_columns);
+  }
+  if (ctx->impl->temporary_options) {
+    grn_options_close(ctx, ctx->impl->temporary_options);
+  }
+  if (ctx->impl->ios) {
+    grn_hash_close(ctx, ctx->impl->ios);
+  }
+  if (ctx->impl->com) {
+    if (ctx->stat != GRN_CTX_QUIT) {
+      int flags;
+      char *str;
+      unsigned int str_len;
+      grn_ctx_send(ctx, "quit", 4, GRN_CTX_HEAD);
+      grn_ctx_recv(ctx, &str, &str_len, &flags);
+    }
+    grn_ctx_send(ctx, "ACK", 3, GRN_CTX_HEAD);
+    rc = grn_com_close(ctx, ctx->impl->com);
+  }
+  GRN_OBJ_FIN(ctx, &ctx->impl->query_log_buf);
+  GRN_OBJ_FIN(ctx, &ctx->impl->output.names);
+  GRN_OBJ_FIN(ctx, &ctx->impl->output.levels);
+  rc = grn_obj_close(ctx, ctx->impl->output.buf);
+  if (ctx->impl->expr_vars) {
+    grn_hash **vp;
+    grn_obj *value;
+    GRN_HASH_EACH(ctx, ctx->impl->expr_vars, eid, NULL, NULL, &vp, {
+      if (*vp) {
+        GRN_HASH_EACH(ctx, *vp, id, NULL, NULL, &value, {
+          GRN_OBJ_FIN(ctx, value);
+        });
+      }
+      grn_hash_close(ctx, *vp);
+    });
+    grn_hash_close(ctx, ctx->impl->expr_vars);
+  }
+  if (ctx->impl->stack) {
+    GRN_FREE(ctx->impl->stack);
+  }
+  if (ctx->impl->db && ctx->flags & GRN_CTX_PER_DB) {
+    grn_obj *db = ctx->impl->db;
+    ctx->impl->db = NULL;
+    grn_obj_close(ctx, db);
+  }
+  grn_alloc_fin_ctx_impl(ctx);
+  grn_alloc_info_dump(ctx);
+  grn_alloc_info_free(ctx);
+  CRITICAL_SECTION_FIN(ctx->impl->lock);
+  {
+    grn_io_mapinfo mi;
+    mi.map = (void *)ctx->impl;
+    grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
+  }
+
+  return rc;
+}
+
 static grn_rc
 grn_ctx_init_internal(grn_ctx *ctx, int flags)
 {
@@ -446,96 +564,7 @@ grn_ctx_fin(grn_ctx *ctx)
     CRITICAL_SECTION_LEAVE(grn_glock);
   }
   if (ctx->impl) {
-    grn_ctx_impl_clear_n_same_error_messagges(ctx);
-    if (ctx->impl->finalizer) {
-      ctx->impl->finalizer(ctx, 0, NULL, &(ctx->user_data));
-    }
-    {
-      grn_obj *stack;
-      grn_obj *spaces;
-      unsigned int i, n_spaces;
-
-      stack = &(ctx->impl->temporary_open_spaces.stack);
-      spaces = (grn_obj *)GRN_BULK_HEAD(stack);
-      n_spaces = GRN_BULK_VSIZE(stack) / sizeof(grn_obj);
-      for (i = 0; i < n_spaces; i++) {
-        grn_obj *space = spaces + (n_spaces - i - 1);
-        GRN_OBJ_FIN(ctx, space);
-      }
-      GRN_OBJ_FIN(ctx, stack);
-    }
-    grn_ctx_impl_mrb_fin(ctx);
-    grn_ctx_loader_clear(ctx);
-    if (ctx->impl->parser) {
-      grn_expr_parser_close(ctx);
-    }
-    GRN_OBJ_FIN(ctx, &ctx->impl->current_request_id);
-    if (ctx->impl->values) {
-#ifndef USE_MEMORY_DEBUG
-      grn_db_obj *o;
-      GRN_ARRAY_EACH(ctx, ctx->impl->values, 0, 0, id, &o, {
-        grn_obj_close(ctx, *((grn_obj **)o));
-      });
-#endif
-      grn_array_close(ctx, ctx->impl->values);
-    }
-    if (ctx->impl->temporary_columns) {
-#ifndef USE_MEMORY_DEBUG
-      grn_obj *value;
-      GRN_PAT_EACH(ctx, ctx->impl->temporary_columns, id, NULL, NULL, &value, {
-        grn_obj_close(ctx, *((grn_obj **)value));
-      });
-#endif
-      grn_pat_close(ctx, ctx->impl->temporary_columns);
-    }
-    if (ctx->impl->ios) {
-      grn_hash_close(ctx, ctx->impl->ios);
-    }
-    if (ctx->impl->com) {
-      if (ctx->stat != GRN_CTX_QUIT) {
-        int flags;
-        char *str;
-        unsigned int str_len;
-        grn_ctx_send(ctx, "quit", 4, GRN_CTX_HEAD);
-        grn_ctx_recv(ctx, &str, &str_len, &flags);
-      }
-      grn_ctx_send(ctx, "ACK", 3, GRN_CTX_HEAD);
-      rc = grn_com_close(ctx, ctx->impl->com);
-    }
-    GRN_OBJ_FIN(ctx, &ctx->impl->query_log_buf);
-    GRN_OBJ_FIN(ctx, &ctx->impl->output.names);
-    GRN_OBJ_FIN(ctx, &ctx->impl->output.levels);
-    rc = grn_obj_close(ctx, ctx->impl->output.buf);
-    if (ctx->impl->expr_vars) {
-      grn_hash **vp;
-      grn_obj *value;
-      GRN_HASH_EACH(ctx, ctx->impl->expr_vars, eid, NULL, NULL, &vp, {
-        if (*vp) {
-          GRN_HASH_EACH(ctx, *vp, id, NULL, NULL, &value, {
-            GRN_OBJ_FIN(ctx, value);
-          });
-        }
-        grn_hash_close(ctx, *vp);
-      });
-      grn_hash_close(ctx, ctx->impl->expr_vars);
-    }
-    if (ctx->impl->stack) {
-      GRN_FREE(ctx->impl->stack);
-    }
-    if (ctx->impl->db && ctx->flags & GRN_CTX_PER_DB) {
-      grn_obj *db = ctx->impl->db;
-      ctx->impl->db = NULL;
-      grn_obj_close(ctx, db);
-    }
-    grn_alloc_fin_ctx_impl(ctx);
-    grn_alloc_info_dump(ctx);
-    grn_alloc_info_free(ctx);
-    CRITICAL_SECTION_FIN(ctx->impl->lock);
-    {
-      grn_io_mapinfo mi;
-      mi.map = (void *)ctx->impl;
-      grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
-    }
+    rc = grn_ctx_impl_fin(ctx);
     ctx->impl = NULL;
   }
   ctx->stat = GRN_CTX_FIN;
@@ -1630,6 +1659,8 @@ exception_filter(EXCEPTION_POINTERS *info)
   DWORD machine_type;
   DWORD previous_address;
 
+  GRN_LOG(ctx, GRN_LOG_CRIT, "-- CRASHED!!! --");
+
   process = GetCurrentProcess();
   thread = GetCurrentThread();
   context = info->ContextRecord;
@@ -1748,19 +1779,28 @@ exception_filter(EXCEPTION_POINTERS *info)
 
     {
       const char *unknown = "(unknown)";
+      const char *grn_encoding_image_name = unknown;
+      if (have_module_name) {
+        grn_encoding_image_name =
+          grn_encoding_convert_from_locale(ctx, module.ImageName, -1, NULL);
+      }
       GRN_LOG(ctx, GRN_LOG_CRIT,
-              "%s:%d:%d: %.*s(): <%s>: <%s>",
+              "%s:%lu:%lu: %.*s(): <%s>: <%s>",
               (have_location ? line.FileName : unknown),
               (have_location ? line.LineNumber : -1),
               (have_location ? line_displacement : -1),
-              (have_symbol_name ? symbol->NameLen : strlen(unknown)),
+              (int)(have_symbol_name ? symbol->NameLen : strlen(unknown)),
               (have_symbol_name ? symbol->Name : unknown),
               (have_module_name ? module.ModuleName : unknown),
-              (have_module_name ? module.ImageName : unknown));
+              grn_encoding_image_name);
+      if (have_module_name) {
+        grn_encoding_converted_free(ctx, grn_encoding_image_name);
+      }
     }
 
     previous_address = address;
   }
+  GRN_LOG(ctx, GRN_LOG_CRIT, "----------------");
 
   return EXCEPTION_CONTINUE_SEARCH;
 }
@@ -1936,7 +1976,7 @@ grn_ctx_output_null(grn_ctx *ctx)
 }
 
 void
-grn_ctx_output_int32(grn_ctx *ctx, int value)
+grn_ctx_output_int32(grn_ctx *ctx, int32_t value)
 {
   grn_output_int32(ctx,
                    ctx->impl->output.buf,
@@ -1944,6 +1984,15 @@ grn_ctx_output_int32(grn_ctx *ctx, int value)
                    value);
 }
 
+void
+grn_ctx_output_uint32(grn_ctx *ctx, uint32_t value)
+{
+  grn_output_uint32(ctx,
+                    ctx->impl->output.buf,
+                    ctx->impl->output.type,
+                    value);
+}
+
 void
 grn_ctx_output_int64(grn_ctx *ctx, int64_t value)
 {
