@@ -1,6 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2011-2017 Brazil
+  Copyright(C) 2011-2018 Brazil
+  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -127,13 +128,29 @@ grn_dat_init(grn_ctx *, grn_dat *dat)
   dat->encoding = GRN_ENC_DEFAULT;
   dat->trie = NULL;
   dat->old_trie = NULL;
-  dat->tokenizer = NULL;
-  dat->normalizer = NULL;
-  GRN_PTR_INIT(&(dat->token_filters), GRN_OBJ_VECTOR, GRN_ID_NIL);
+  GRN_TEXT_INIT(&(dat->token_filters), 0);
+  GRN_PTR_INIT(&(dat->token_filter_procs), GRN_OBJ_VECTOR, GRN_ID_NIL);
   CRITICAL_SECTION_INIT(dat->lock);
   dat->is_dirty = GRN_FALSE;
 }
 
+static void
+grn_dat_close_token_filters(grn_ctx *ctx, grn_dat *dat)
+{
+  grn_obj *token_filters = &(dat->token_filters);
+  grn_table_module *raw_token_filters =
+    (grn_table_module *)GRN_BULK_HEAD(token_filters);
+  size_t i, n;
+
+  n = GRN_BULK_VSIZE(token_filters) / sizeof(grn_table_module);
+  for (i = 0; i < n; i++) {
+    grn_table_module *raw_token_filter = raw_token_filters + i;
+    grn_table_module_fin(ctx, raw_token_filter);
+  }
+  GRN_OBJ_FIN(ctx, token_filters);
+  GRN_OBJ_FIN(ctx, &(dat->token_filter_procs));
+}
+
 void
 grn_dat_fin(grn_ctx *ctx, grn_dat *dat)
 {
@@ -150,7 +167,9 @@ grn_dat_fin(grn_ctx *ctx, grn_dat *dat)
     grn_io_close(ctx, dat->io);
     dat->io = NULL;
   }
-  GRN_OBJ_FIN(ctx, &(dat->token_filters));
+  grn_table_module_fin(ctx, &(dat->tokenizer));
+  grn_table_module_fin(ctx, &(dat->normalizer));
+  grn_dat_close_token_filters(ctx, dat);
 }
 
 /*
@@ -246,6 +265,7 @@ grn_dat_open_trie_if_needed(grn_ctx *ctx, grn_dat *dat)
 }
 
 bool grn_dat_rebuild_trie(grn_ctx *ctx, grn_dat *dat) {
+  const grn::dat::Trie * const trie = static_cast<grn::dat::Trie *>(dat->trie);
   grn::dat::Trie * const new_trie = new (std::nothrow) grn::dat::Trie;
   if (!new_trie) {
     MERR("new grn::dat::Trie failed");
@@ -253,17 +273,22 @@ bool grn_dat_rebuild_trie(grn_ctx *ctx, grn_dat *dat) {
   }
 
   const uint32_t file_id = dat->header->file_id;
-  try {
-    char trie_path[PATH_MAX];
-    grn_dat_generate_trie_path(grn_io_path(dat->io), trie_path, file_id + 1);
-    const grn::dat::Trie * const trie = static_cast<grn::dat::Trie *>(dat->trie);
-    new_trie->create(*trie, trie_path, trie->file_size() * 2);
-  } catch (const grn::dat::Exception &ex) {
-    ERR(grn_dat_translate_error_code(ex.code()),
-        "grn::dat::Trie::open failed: %s",
-        ex.what());
-    delete new_trie;
-    return false;
+  char trie_path[PATH_MAX];
+  grn_dat_generate_trie_path(grn_io_path(dat->io), trie_path, file_id + 1);
+
+  for (uint64_t file_size = trie->file_size() * 2;; file_size *= 2) {
+    try {
+      new_trie->create(*trie, trie_path, file_size);
+    } catch (const grn::dat::SizeError &) {
+      continue;
+    } catch (const grn::dat::Exception &ex) {
+      ERR(grn_dat_translate_error_code(ex.code()),
+          "grn::dat::Trie::open failed: %s",
+          ex.what());
+      delete new_trie;
+      return false;
+    }
+    break;
   }
 
   grn::dat::Trie * const old_trie = static_cast<grn::dat::Trie *>(dat->old_trie);
@@ -342,15 +367,16 @@ grn_dat_create(grn_ctx *ctx, const char *path, uint32_t,
   dat->header->file_id = 0;
   if (dat->header->flags & GRN_OBJ_KEY_NORMALIZE) {
     dat->header->flags &= ~GRN_OBJ_KEY_NORMALIZE;
-    dat->normalizer = grn_ctx_get(ctx, GRN_NORMALIZER_AUTO_NAME, -1);
-    dat->header->normalizer = grn_obj_id(ctx, dat->normalizer);
+    grn_obj *normalizer = grn_ctx_get(ctx, GRN_NORMALIZER_AUTO_NAME, -1);
+    dat->header->normalizer = grn_obj_id(ctx, normalizer);
   } else {
-    dat->normalizer = NULL;
     dat->header->normalizer = GRN_ID_NIL;
   }
   dat->encoding = encoding;
-  dat->tokenizer = NULL;
-  GRN_PTR_INIT(&(dat->token_filters), GRN_OBJ_VECTOR, GRN_ID_NIL);
+  grn_table_module_init(ctx, &(dat->tokenizer), GRN_ID_NIL);
+  grn_table_module_init(ctx, &(dat->normalizer), dat->header->normalizer);
+  GRN_TEXT_INIT(&(dat->token_filters), 0);
+  GRN_PTR_INIT(&(dat->token_filter_procs), GRN_OBJ_VECTOR, GRN_ID_NIL);
 
   dat->obj.header.flags = dat->header->flags;
 
@@ -385,15 +411,15 @@ grn_dat_open(grn_ctx *ctx, const char *path)
   }
   dat->file_id = dat->header->file_id;
   dat->encoding = dat->header->encoding;
-  dat->tokenizer = grn_ctx_at(ctx, dat->header->tokenizer);
+  grn_table_module_init(ctx, &(dat->tokenizer), dat->header->tokenizer);
   if (dat->header->flags & GRN_OBJ_KEY_NORMALIZE) {
     dat->header->flags &= ~GRN_OBJ_KEY_NORMALIZE;
-    dat->normalizer = grn_ctx_get(ctx, GRN_NORMALIZER_AUTO_NAME, -1);
-    dat->header->normalizer = grn_obj_id(ctx, dat->normalizer);
-  } else {
-    dat->normalizer = grn_ctx_at(ctx, dat->header->normalizer);
+    grn_obj *normalizer = grn_ctx_get(ctx, GRN_NORMALIZER_AUTO_NAME, -1);
+    dat->header->normalizer = grn_obj_id(ctx, normalizer);
   }
-  GRN_PTR_INIT(&(dat->token_filters), GRN_OBJ_VECTOR, GRN_ID_NIL);
+  grn_table_module_init(ctx, &(dat->normalizer), dat->header->normalizer);
+  GRN_TEXT_INIT(&(dat->token_filters), 0);
+  GRN_PTR_INIT(&(dat->token_filter_procs), GRN_OBJ_VECTOR, GRN_ID_NIL);
   dat->obj.header.flags = dat->header->flags;
   return dat;
 }
@@ -744,10 +770,10 @@ grn_dat_scan(grn_ctx *ctx, grn_dat *dat, const char *str,
 
   unsigned int num_scan_hits = 0;
   try {
-    if (dat->normalizer) {
+    if (dat->normalizer.proc) {
       int flags = GRN_STRING_WITH_CHECKS;
       grn_obj * const normalized_string = grn_string_open(ctx, str, str_size,
-                                                          dat->normalizer,
+                                                          dat->normalizer.proc,
                                                           flags);
       if (!normalized_string) {
         if (str_rest) {
