@@ -387,11 +387,10 @@ typedef int grn_cond;
 # define GRN_FMT_SSIZE "Id"
 # ifdef WIN64
 #  define GRN_FMT_SOCKET GRN_FMT_INT64U
-#  define GRN_FMT_DWORD  "lu"
 # else /* WIN64 */
 #  define GRN_FMT_SOCKET GRN_FMT_INT32U
-#  define GRN_FMT_DWORD  "u"
 # endif /* WIN64 */
+# define GRN_FMT_DWORD "lu"
 # define GRN_FMT_OFF64_T GRN_FMT_LLD
 #else /* WIN32 */
 # define GRN_FMT_LLD  "lld"
@@ -679,6 +678,10 @@ grn_str_greater(const uint8_t *ap, uint32_t as, const uint8_t *bp, uint32_t bs)
 
 #define GRN_NEXT_ADDR(p) (((byte *)(p)) + sizeof(*(p)))
 
+#ifdef GRN_WITH_ONIGMO
+# define GRN_SUPPORT_REGEXP
+#endif /* GRN_WITH_ONIGMO */
+
 GRN_API void grn_sleep(uint32_t seconds);
 GRN_API void grn_nanosleep(uint64_t nanoseconds);
 
