@@ -1,5 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2010-2013 Brazil
+/*
+  Copyright(C) 2010-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -14,8 +15,8 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#ifndef GRN_PLUGIN_H
-#define GRN_PLUGIN_H
+
+#pragma once
 
 #include "grn.h"
 #include "grn_ctx.h"
@@ -37,6 +38,7 @@ typedef void * grn_dl_symbol;
 typedef struct _grn_plugin grn_plugin;
 
 struct _grn_plugin {
+  char path[PATH_MAX];
   grn_dl dl;
   grn_plugin_func init_func;
   grn_plugin_func register_func;
@@ -45,6 +47,7 @@ struct _grn_plugin {
   int refcount;
 };
 
+void grn_plugin_init_from_env(void);
 grn_rc grn_plugins_init(void);
 grn_rc grn_plugins_fin(void);
 grn_id grn_plugin_open(grn_ctx *ctx, const char *filename);
@@ -57,5 +60,3 @@ void grn_plugin_ensure_registered(grn_ctx *ctx, grn_obj *proc);
 #ifdef __cplusplus
 }
 #endif
-
-#endif /* GRN_PLUGIN_H */
