@@ -1,6 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2009-2017 Brazil
+  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -16,9 +17,10 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "grn_load.h"
 #include "grn_ctx_impl.h"
 #include "grn_db.h"
+#include "grn_load.h"
+#include "grn_obj.h"
 #include "grn_util.h"
 
 static void
@@ -307,23 +309,84 @@ parse_id_value(grn_ctx *ctx, grn_obj *value)
   }
 }
 
+static void
+grn_loader_apply_each(grn_ctx *ctx,
+                      grn_loader *loader,
+                      grn_id id)
+{
+  grn_obj *var;
+
+  if (!loader->each) {
+    return;
+  }
+
+  var = grn_expr_get_var_by_offset(ctx, loader->each, 0);
+  GRN_RECORD_SET(ctx, var, id);
+  grn_expr_exec(ctx, loader->each, 0);
+}
+
+static void
+bracket_close_set_values(grn_ctx *ctx,
+                         grn_loader *loader,
+                         grn_id id,
+                         grn_obj *key,
+                         grn_obj *values,
+                         uint32_t n_values)
+{
+  uint32_t i;
+  grn_obj *value;
+  grn_obj **columns; /* Columns except _id and _key. */
+
+  columns = (grn_obj **)GRN_BULK_HEAD(&loader->columns);
+  for (i = 0, value = values;
+       i < n_values;
+       i++, value = values_next(ctx, value)) {
+    grn_obj *column;
+
+    if (i == loader->id_offset || i == loader->key_offset) {
+      /* Skip _id and _key, because it's already used to get id. */
+       continue;
+    }
+    column = *columns;
+    if (value->header.domain == GRN_JSON_LOAD_OPEN_BRACKET) {
+      set_vector(ctx, column, id, value);
+    } else if (value->header.domain == GRN_JSON_LOAD_OPEN_BRACE) {
+      set_weight_vector(ctx, column, id, value);
+    } else {
+      grn_obj_set_value(ctx, column, id, value, GRN_OBJ_SET);
+    }
+    if (ctx->rc != GRN_SUCCESS) {
+      char column_name[GRN_TABLE_MAX_KEY_SIZE];
+      unsigned int column_name_size;
+      grn_loader_save_error(ctx, loader);
+      column_name_size = grn_obj_name(ctx, column, column_name,
+                                      GRN_TABLE_MAX_KEY_SIZE);
+      report_set_column_value_failure(ctx, key,
+                                      column_name, column_name_size,
+                                      value);
+      loader->n_column_errors++;
+      ERRCLR(ctx);
+    }
+    columns++;
+  }
+}
+
 static void
 bracket_close(grn_ctx *ctx, grn_loader *loader)
 {
   grn_id id = GRN_ID_NIL;
-  grn_obj *value, *value_end, *id_value = NULL, *key_value = NULL;
-  grn_obj *col, **cols; /* Columns except _id and _key. */
+  grn_obj *value_begin, *value_end;
+  grn_obj *value;
+  grn_obj *key = NULL;
   uint32_t i, begin;
-  uint32_t ncols;   /* Number of columns except _id and _key. */
   uint32_t nvalues; /* Number of values in brackets. */
   uint32_t depth;
   grn_bool is_record_load = GRN_FALSE;
 
-  cols = (grn_obj **)GRN_BULK_HEAD(&loader->columns);
-  ncols = GRN_BULK_VSIZE(&loader->columns) / sizeof(grn_obj *);
   GRN_UINT32_POP(&loader->level, begin);
-  value = (grn_obj *)GRN_TEXT_VALUE(&loader->values) + begin;
+  value_begin = (grn_obj *)GRN_TEXT_VALUE(&loader->values) + begin;
   value_end = (grn_obj *)GRN_TEXT_VALUE(&loader->values) + loader->values_size;
+  value = value_begin;
   GRN_ASSERT(value->header.domain == GRN_JSON_LOAD_OPEN_BRACKET);
   GRN_UINT32_SET(ctx, value, loader->values_size - begin - 1);
   value++;
@@ -345,6 +408,7 @@ bracket_close(grn_ctx *ctx, grn_loader *loader)
     for (i = 0; i < nvalues; i++) {
       const char *col_name;
       unsigned int col_name_size;
+      grn_obj *col;
       if (value->header.domain != GRN_DB_TEXT) {
         grn_obj buffer;
         GRN_TEXT_INIT(&buffer, 0);
@@ -438,7 +502,8 @@ bracket_close(grn_ctx *ctx, grn_loader *loader)
      */
     id = grn_table_add(ctx, loader->table, NULL, 0, NULL);
   } else {
-    uint32_t expected_nvalues = ncols;
+    uint32_t expected_nvalues =
+      GRN_BULK_VSIZE(&loader->columns) / sizeof(grn_obj *);
     if (loader->id_offset != -1 || loader->key_offset != -1) {
       expected_nvalues++;
     }
@@ -450,14 +515,14 @@ bracket_close(grn_ctx *ctx, grn_loader *loader)
       goto exit;
     }
     if (loader->id_offset != -1) {
-      id_value = value + loader->id_offset;
-      id = parse_id_value(ctx, id_value);
+      grn_obj *id_bulk = value + loader->id_offset;
+      id = parse_id_value(ctx, id_bulk);
       if (grn_table_at(ctx, loader->table, id) == GRN_ID_NIL) {
         id = grn_table_add(ctx, loader->table, NULL, 0, NULL);
       }
     } else if (loader->key_offset != -1) {
-      key_value = value + loader->key_offset;
-      id = loader_add(ctx, key_value);
+      key = value + loader->key_offset;
+      id = loader_add(ctx, key);
     } else {
       id = grn_table_add(ctx, loader->table, NULL, 0, NULL);
     }
@@ -467,37 +532,16 @@ bracket_close(grn_ctx *ctx, grn_loader *loader)
     goto exit;
   }
 
-  for (i = 0; i < nvalues; i++, value = values_next(ctx, value)) {
-    if (i == loader->id_offset || i == loader->key_offset) {
-       /* Skip _id and _key, because it's already used to get id. */
-       continue;
-    }
-    col = *cols;
-    if (value->header.domain == GRN_JSON_LOAD_OPEN_BRACKET) {
-      set_vector(ctx, col, id, value);
-    } else if (value->header.domain == GRN_JSON_LOAD_OPEN_BRACE) {
-      set_weight_vector(ctx, col, id, value);
-    } else {
-      grn_obj_set_value(ctx, col, id, value, GRN_OBJ_SET);
-    }
-    if (ctx->rc != GRN_SUCCESS) {
-      char column_name[GRN_TABLE_MAX_KEY_SIZE];
-      unsigned int column_name_size;
-      grn_loader_save_error(ctx, loader);
-      column_name_size = grn_obj_name(ctx, col, column_name,
-                                      GRN_TABLE_MAX_KEY_SIZE);
-      report_set_column_value_failure(ctx, key_value,
-                                      column_name, column_name_size,
-                                      value);
-      loader->n_column_errors++;
-      ERRCLR(ctx);
-    }
-    cols++;
-  }
-  if (loader->each) {
-    grn_obj *v = grn_expr_get_var_by_offset(ctx, loader->each, 0);
-    GRN_RECORD_SET(ctx, v, id);
-    grn_expr_exec(ctx, loader->each, 0);
+  if (loader->lock_table) {
+    GRN_TABLE_LOCK_BEGIN(ctx, loader->table) {
+      if (grn_table_at(ctx, loader->table, id) == id) {
+        bracket_close_set_values(ctx, loader, id, key, value, nvalues);
+        grn_loader_apply_each(ctx, loader, id);
+      }
+    } GRN_TABLE_LOCK_END(ctx, table);
+  } else {
+    bracket_close_set_values(ctx, loader, id, key, value, nvalues);
+    grn_loader_apply_each(ctx, loader, id);
   }
   loader->nrecords++;
 exit:
@@ -522,12 +566,79 @@ exit:
   ERRCLR(ctx);
 }
 
+static void
+brace_close_set_values(grn_ctx *ctx,
+                       grn_loader *loader,
+                       grn_id id,
+                       grn_obj *key,
+                       grn_obj *values_begin,
+                       grn_obj *values_end)
+{
+  grn_obj *value;
+
+  for (value = values_begin;
+       value + 1 < values_end;
+       value = values_next(ctx, value)) {
+    const char *name = GRN_TEXT_VALUE(value);
+    unsigned int name_size = GRN_TEXT_LEN(value);
+    grn_obj *column;
+    value++;
+    if (name_equal(name, name_size, GRN_COLUMN_NAME_ID) ||
+        name_equal(name, name_size, GRN_COLUMN_NAME_KEY)) {
+      /* Skip _id and _key, because it's already used to get id. */
+      continue;
+    }
+    column = grn_obj_column(ctx, loader->table, name, name_size);
+    if (!column) {
+      char table_name[GRN_TABLE_MAX_KEY_SIZE];
+      int table_name_size;
+      table_name_size = grn_obj_name(ctx,
+                                     loader->table,
+                                     table_name,
+                                     GRN_TABLE_MAX_KEY_SIZE);
+      GRN_LOG(ctx, GRN_LOG_ERROR, "[load] nonexistent column: <%.*s.%.*s>",
+              table_name_size,
+              table_name,
+              (int)name_size,
+              name);
+      /* Automatic column creation is disabled. */
+      /*
+      if (value->header.domain == GRN_JSON_LOAD_OPEN_BRACKET) {
+        grn_obj *v = value + 1;
+        col = grn_column_create(ctx, loader->table, name, name_size,
+                                NULL, GRN_OBJ_PERSISTENT|GRN_OBJ_COLUMN_VECTOR,
+                                grn_ctx_at(ctx, v->header.domain));
+      } else {
+        col = grn_column_create(ctx, loader->table, name, name_size,
+                                NULL, GRN_OBJ_PERSISTENT,
+                                grn_ctx_at(ctx, value->header.domain));
+      }
+      */
+    } else {
+      if (value->header.domain == GRN_JSON_LOAD_OPEN_BRACKET) {
+        set_vector(ctx, column, id, value);
+      } else if (value->header.domain == GRN_JSON_LOAD_OPEN_BRACE) {
+        set_weight_vector(ctx, column, id, value);
+      } else {
+        grn_obj_set_value(ctx, column, id, value, GRN_OBJ_SET);
+      }
+      if (ctx->rc != GRN_SUCCESS) {
+        grn_loader_save_error(ctx, loader);
+        report_set_column_value_failure(ctx, key, name, name_size, value);
+        loader->n_column_errors++;
+        ERRCLR(ctx);
+      }
+      grn_obj_unlink(ctx, column);
+    }
+  }
+}
+
 static void
 brace_close(grn_ctx *ctx, grn_loader *loader)
 {
   grn_id id = GRN_ID_NIL;
   grn_obj *value, *value_begin, *value_end;
-  grn_obj *id_value = NULL, *key_value = NULL;
+  grn_obj *id_bulk = NULL, *key = NULL;
   uint32_t begin;
 
   GRN_UINT32_POP(&loader->level, begin);
@@ -560,28 +671,28 @@ brace_close(grn_ctx *ctx, grn_loader *loader)
     }
     value++;
     if (name_equal(name, name_size, GRN_COLUMN_NAME_ID)) {
-      if (id_value || key_value) {
+      if (id_bulk || key) {
         if (loader->table->header.type == GRN_TABLE_NO_KEY) {
           GRN_LOG(ctx, GRN_LOG_ERROR, "duplicated '_id' column");
           goto exit;
         } else {
           GRN_LOG(ctx, GRN_LOG_ERROR,
                   "duplicated key columns: %s and %s",
-                  id_value ? GRN_COLUMN_NAME_ID : GRN_COLUMN_NAME_KEY,
+                  id_bulk ? GRN_COLUMN_NAME_ID : GRN_COLUMN_NAME_KEY,
                   GRN_COLUMN_NAME_ID);
           goto exit;
         }
       }
-      id_value = value;
+      id_bulk = value;
     } else if (name_equal(name, name_size, GRN_COLUMN_NAME_KEY)) {
-      if (id_value || key_value) {
+      if (id_bulk || key) {
         GRN_LOG(ctx, GRN_LOG_ERROR,
                 "duplicated key columns: %s and %s",
-                id_value ? GRN_COLUMN_NAME_ID : GRN_COLUMN_NAME_KEY,
+                id_bulk ? GRN_COLUMN_NAME_ID : GRN_COLUMN_NAME_KEY,
                 GRN_COLUMN_NAME_KEY);
         goto exit;
       }
-      key_value = value;
+      key = value;
     }
   }
 
@@ -590,29 +701,29 @@ brace_close(grn_ctx *ctx, grn_loader *loader)
   case GRN_TABLE_PAT_KEY :
   case GRN_TABLE_DAT_KEY :
     /* The target table requires _id or _key. */
-    if (!id_value && !key_value) {
+    if (!id_bulk && !key) {
       GRN_LOG(ctx, GRN_LOG_ERROR, "neither _key nor _id is assigned");
       goto exit;
     }
     break;
   default :
     /* The target table does not have _key. */
-    if (key_value) {
+    if (key) {
       GRN_LOG(ctx, GRN_LOG_ERROR, "nonexistent key value");
       goto exit;
     }
     break;
   }
 
-  if (id_value) {
-    id = parse_id_value(ctx, id_value);
+  if (id_bulk) {
+    id = parse_id_value(ctx, id_bulk);
     if (grn_table_at(ctx, loader->table, id) == GRN_ID_NIL) {
       if (ctx->rc == GRN_SUCCESS) {
         id = grn_table_add(ctx, loader->table, NULL, 0, NULL);
       }
     }
-  } else if (key_value) {
-    id = loader_add(ctx, key_value);
+  } else if (key) {
+    id = loader_add(ctx, key);
   } else {
     id = grn_table_add(ctx, loader->table, NULL, 0, NULL);
   }
@@ -621,55 +732,16 @@ brace_close(grn_ctx *ctx, grn_loader *loader)
     goto exit;
   }
 
-  for (value = value_begin; value + 1 < value_end;
-       value = values_next(ctx, value)) {
-    grn_obj *col;
-    const char *name = GRN_TEXT_VALUE(value);
-    unsigned int name_size = GRN_TEXT_LEN(value);
-    value++;
-    if (value == id_value || value == key_value) {
-      /* Skip _id and _key, because it's already used to get id. */
-      continue;
-    }
-    col = grn_obj_column(ctx, loader->table, name, name_size);
-    if (!col) {
-      GRN_LOG(ctx, GRN_LOG_ERROR, "invalid column('%.*s')",
-              (int)name_size, name);
-      /* Automatic column creation is disabled. */
-      /*
-      if (value->header.domain == GRN_JSON_LOAD_OPEN_BRACKET) {
-        grn_obj *v = value + 1;
-        col = grn_column_create(ctx, loader->table, name, name_size,
-                                NULL, GRN_OBJ_PERSISTENT|GRN_OBJ_COLUMN_VECTOR,
-                                grn_ctx_at(ctx, v->header.domain));
-      } else {
-        col = grn_column_create(ctx, loader->table, name, name_size,
-                                NULL, GRN_OBJ_PERSISTENT,
-                                grn_ctx_at(ctx, value->header.domain));
+  if (loader->lock_table) {
+    GRN_TABLE_LOCK_BEGIN(ctx, loader->table) {
+      if (grn_table_at(ctx, loader->table, id) == id) {
+        brace_close_set_values(ctx, loader, id, key, value_begin, value_end);
+        grn_loader_apply_each(ctx, loader, id);
       }
-      */
-    } else {
-      if (value->header.domain == GRN_JSON_LOAD_OPEN_BRACKET) {
-        set_vector(ctx, col, id, value);
-      } else if (value->header.domain == GRN_JSON_LOAD_OPEN_BRACE) {
-        set_weight_vector(ctx, col, id, value);
-      } else {
-        grn_obj_set_value(ctx, col, id, value, GRN_OBJ_SET);
-      }
-      if (ctx->rc != GRN_SUCCESS) {
-        grn_loader_save_error(ctx, loader);
-        report_set_column_value_failure(ctx, key_value,
-                                        name, name_size, value);
-        loader->n_column_errors++;
-        ERRCLR(ctx);
-      }
-      grn_obj_unlink(ctx, col);
-    }
-  }
-  if (loader->each) {
-    value = grn_expr_get_var_by_offset(ctx, loader->each, 0);
-    GRN_RECORD_SET(ctx, value, id);
-    grn_expr_exec(ctx, loader->each, 0);
+    } GRN_TABLE_LOCK_END(ctx, loader->table);
+  } else {
+    brace_close_set_values(ctx, loader, id, key, value_begin, value_end);
+    grn_loader_apply_each(ctx, loader, id);
   }
   loader->nrecords++;
 exit:
@@ -1003,13 +1075,27 @@ json_read(grn_ctx *ctx, grn_loader *loader, const char *str, unsigned int str_le
       }
       {
         uint32_t u = loader->unichar;
+        if (u >= 0xd800 && u <= 0xdbff) { /* High-surrogate code points */
+          loader->unichar_hi = u;
+          loader->stat = GRN_LOADER_STRING;
+          str++;
+          break;
+        }
+        if (u >= 0xdc00 && u <= 0xdfff) { /* Low-surrogate code points */
+          u = 0x10000 + (loader->unichar_hi - 0xd800) * 0x400 + u - 0xdc00;
+        }
         if (u < 0x80) {
           GRN_TEXT_PUTC(ctx, loader->last, u);
         } else {
           if (u < 0x800) {
-            GRN_TEXT_PUTC(ctx, loader->last, ((u >> 6) & 0x1f) | 0xc0);
+            GRN_TEXT_PUTC(ctx, loader->last, (u >> 6) | 0xc0);
           } else {
-            GRN_TEXT_PUTC(ctx, loader->last, (u >> 12) | 0xe0);
+            if (u < 0x10000) {
+              GRN_TEXT_PUTC(ctx, loader->last, (u >> 12) | 0xe0);
+            } else {
+              GRN_TEXT_PUTC(ctx, loader->last, (u >> 18) | 0xf0);
+              GRN_TEXT_PUTC(ctx, loader->last, ((u >> 12) & 0x3f) | 0x80);
+            }
             GRN_TEXT_PUTC(ctx, loader->last, ((u >> 6) & 0x3f) | 0x80);
           }
           GRN_TEXT_PUTC(ctx, loader->last, (u & 0x3f) | 0x80);
@@ -1179,6 +1265,7 @@ grn_load_internal(grn_ctx *ctx, grn_load_input *input)
     }
     loader->output_ids = input->output_ids;
     loader->output_errors = input->output_errors;
+    loader->lock_table = input->lock_table;
   } else {
     if (!loader->table) {
       ERR(GRN_INVALID_ARGUMENT, "mandatory \"table\" parameter is absent");
@@ -1230,6 +1317,7 @@ grn_load(grn_ctx *ctx, grn_content_type input_type,
     input.each.length = each_len;
     input.output_ids = GRN_FALSE;
     input.output_errors = GRN_FALSE;
+    input.lock_table = GRN_FALSE;
     input.emit_level = 1;
     grn_load_internal(ctx, &input);
   }
