@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2013 Brazil
+  Copyright(C) 2013-2015 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "../ctx_impl.h"
+#include "../grn_ctx_impl.h"
 
 #ifdef GRN_WITH_MRUBY
 #include <mruby.h>
@@ -24,28 +24,16 @@
 #include <mruby/variable.h>
 #include <mruby/data.h>
 
-#include "../db.h"
+#include "../grn_db.h"
+#include "mrb_ctx.h"
 #include "mrb_accessor.h"
+#include "mrb_converter.h"
 
 static struct mrb_data_type mrb_grn_accessor_type = {
   "Groonga::Accessor",
   NULL
 };
 
-mrb_value
-mrb_grn_accessor_new(mrb_state *mrb, grn_accessor *accessor)
-{
-  grn_ctx *ctx = (grn_ctx *)mrb->ud;
-  struct RClass *module = ctx->impl->mrb.module;
-  struct RClass *klass;
-  mrb_value mrb_accessor_ptr;
-
-  mrb_accessor_ptr = mrb_cptr_value(mrb, accessor);
-  klass = mrb_class_ptr(mrb_const_get(mrb, mrb_obj_value(module),
-                                      mrb_intern(mrb, "Accessor")));
-  return mrb_obj_new(mrb, klass, 1, &mrb_accessor_ptr);
-}
-
 static mrb_value
 mrb_grn_accessor_initialize(mrb_state *mrb, mrb_value self)
 {
@@ -63,8 +51,49 @@ mrb_grn_accessor_next(mrb_state *mrb, mrb_value self)
   grn_accessor *accessor;
 
   accessor = DATA_PTR(self);
-  if (!accessor->next) { return mrb_nil_value(); }
-  return mrb_cptr_value(mrb, accessor->next);
+  return grn_mrb_value_from_grn_obj(mrb, (grn_obj *)(accessor->next));
+}
+
+static mrb_value
+mrb_grn_accessor_have_next_p(mrb_state *mrb, mrb_value self)
+{
+  grn_accessor *accessor;
+
+  accessor = DATA_PTR(self);
+  return mrb_bool_value(accessor->next != NULL);
+}
+
+static mrb_value
+mrb_grn_accessor_object(mrb_state *mrb, mrb_value self)
+{
+  grn_accessor *accessor;
+
+  accessor = DATA_PTR(self);
+  return grn_mrb_value_from_grn_obj(mrb, accessor->obj);
+}
+
+static mrb_value
+mrb_grn_accessor_name(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  grn_rc rc;
+  grn_obj *accessor;
+  grn_obj name;
+  mrb_value mrb_name;
+
+  accessor = DATA_PTR(self);
+  GRN_TEXT_INIT(&name, 0);
+  rc = grn_column_name_(ctx, accessor, &name);
+  if (rc == GRN_SUCCESS) {
+    mrb_name = mrb_str_new(mrb, GRN_TEXT_VALUE(&name), GRN_TEXT_LEN(&name));
+    GRN_OBJ_FIN(ctx, &name);
+  } else {
+    mrb_name = mrb_nil_value();
+    GRN_OBJ_FIN(ctx, &name);
+    grn_mrb_ctx_check(mrb);
+  }
+
+  return mrb_name;
 }
 
 void
@@ -81,5 +110,12 @@ grn_mrb_accessor_init(grn_ctx *ctx)
                     mrb_grn_accessor_initialize, MRB_ARGS_REQ(1));
   mrb_define_method(mrb, klass, "next",
                     mrb_grn_accessor_next, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "have_next?",
+                    mrb_grn_accessor_have_next_p, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "object",
+                    mrb_grn_accessor_object, MRB_ARGS_NONE());
+
+  mrb_define_method(mrb, klass, "name",
+                    mrb_grn_accessor_name, MRB_ARGS_NONE());
 }
 #endif
