@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2013-2016 Brazil
+  Copyright(C) 2013-2018 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,8 @@
 #include "mrb_options.h"
 #include "mrb_converter.h"
 
+#include "../grn_encoding.h"
+
 static mrb_value
 object_remove_force(mrb_state *mrb, mrb_value self)
 {
@@ -115,7 +117,15 @@ object_get_path(mrb_state *mrb, mrb_value self)
   path = grn_obj_path(ctx, object);
 
   if (path) {
-    return mrb_str_new_cstr(mrb, path);
+    const char *utf8_path;
+    mrb_value mrb_path;
+
+    utf8_path =
+      grn_encoding_convert_to_utf8_from_locale(ctx, path, -1, NULL);
+    mrb_path = mrb_str_new_cstr(mrb, utf8_path);
+    grn_encoding_converted_free(ctx, utf8_path);
+
+    return mrb_path;
   } else {
     return mrb_nil_value();
   }
@@ -161,8 +171,8 @@ object_hash(mrb_state *mrb, mrb_value self)
   return mrb_fixnum_value((mrb_int)((uint64_t)object));
 }
 
-static mrb_value
-object_close(mrb_state *mrb, mrb_value self)
+mrb_value
+grn_mrb_object_close(mrb_state *mrb, mrb_value self)
 {
   grn_ctx *ctx = (grn_ctx *)mrb->ud;
   grn_obj *object;
@@ -206,8 +216,8 @@ object_remove(mrb_state *mrb, mrb_value self)
   return mrb_nil_value();
 }
 
-static mrb_value
-object_is_closed(mrb_state *mrb, mrb_value self)
+mrb_value
+grn_mrb_object_is_closed(mrb_state *mrb, mrb_value self)
 {
   grn_obj *object;
 
@@ -280,6 +290,19 @@ object_is_true(mrb_state *mrb, mrb_value self)
   return mrb_bool_value(grn_obj_is_true(ctx, object));
 }
 
+static mrb_value
+object_check_corrupt(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  grn_obj *object;
+  grn_bool is_corrupt;
+
+  object = DATA_PTR(self);
+  is_corrupt = grn_obj_is_corrupt(ctx, object);
+  grn_mrb_ctx_check(mrb);
+  return mrb_bool_value(is_corrupt);
+}
+
 void
 grn_mrb_object_init(grn_ctx *ctx)
 {
@@ -309,9 +332,11 @@ grn_mrb_object_init(grn_ctx *ctx)
   mrb_define_method(mrb, klass, "==", object_equal, MRB_ARGS_REQ(1));
   mrb_define_method(mrb, klass, "eql?", object_equal, MRB_ARGS_REQ(1));
   mrb_define_method(mrb, klass, "hash", object_hash, MRB_ARGS_NONE());
-  mrb_define_method(mrb, klass, "close", object_close, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "close",
+                    grn_mrb_object_close, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "remove", object_remove, MRB_ARGS_OPT(1));
-  mrb_define_method(mrb, klass, "closed?", object_is_closed, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "closed?",
+                    grn_mrb_object_is_closed, MRB_ARGS_NONE());
 
   mrb_define_method(mrb, klass, "domain_id", object_get_domain_id,
                     MRB_ARGS_NONE());
@@ -325,6 +350,9 @@ grn_mrb_object_init(grn_ctx *ctx)
 
   mrb_define_method(mrb, klass, "true?", object_is_true, MRB_ARGS_NONE());
 
+  mrb_define_method(mrb, klass, "check_corrupt", object_check_corrupt,
+                    MRB_ARGS_NONE());
+
   grn_mrb_load(ctx, "index_info.rb");
 }
 #endif
