@@ -5,6 +5,7 @@ module Groonga
     class Grndb
       def initialize(argv)
         @program_path, *@arguments = argv
+        @output = LocaleOutput.new($stderr)
         @succeeded = true
         @database_path = nil
       end
@@ -15,9 +16,9 @@ module Groonga
         begin
           options = command_line_parser.parse(@arguments)
         rescue Slop::Error => error
-          $stderr.puts(error.message)
-          $stderr.puts
-          $stderr.puts(command_line_parser.help_message)
+          @output.puts(error.message)
+          @output.puts
+          @output.puts(command_line_parser.help_message)
           return false
         end
         @succeeded
@@ -68,9 +69,9 @@ module Groonga
       def open_database(command, options)
         arguments = options.arguments
         if arguments.empty?
-          $stderr.puts("Database path is missing")
-          $stderr.puts
-          $stderr.puts(command.help_message)
+          @output.puts("Database path is missing")
+          @output.puts
+          @output.puts(command.help_message)
           @succeesed = false
           return
         end
@@ -80,8 +81,8 @@ module Groonga
         begin
           database = Database.open(@database_path)
         rescue Error => error
-          $stderr.puts("Failed to open database: <#{@database_path}>")
-          $stderr.puts(error.message)
+          @output.puts("Failed to open database: <#{@database_path}>")
+          @output.puts(error.message)
           @succeeded = false
           return
         end
@@ -95,13 +96,13 @@ module Groonga
 
       def failed(*messages)
         messages.each do |message|
-          $stderr.puts(message)
+          @output.puts(message)
         end
         @succeeded = false
       end
 
       def recover(database, options, arguments)
-        recoverer = Recoverer.new
+        recoverer = Recoverer.new(@output)
         recoverer.database = database
         recoverer.force_truncate = options[:force_truncate]
         recoverer.force_lock_clear = options[:force_lock_clear]
@@ -114,7 +115,7 @@ module Groonga
       end
 
       def check(database, options, arguments)
-        checker = Checker.new
+        checker = Checker.new(@output)
         checker.program_path = @program_path
         checker.database_path = @database_path
         checker.database = database
@@ -142,7 +143,8 @@ module Groonga
         attr_writer :log_paths
         attr_writer :on_failure
 
-        def initialize
+        def initialize(output)
+          @output = output
           @context = Context.instance
           @checked = {}
         end
@@ -210,6 +212,8 @@ module Groonga
 
         private
         def check_log_file(log_file)
+          return # Disable for now
+
           parser = GroongaLog::Parser.new
           parser.parse(log_file) do |statistic|
             p statistic.to_h
@@ -451,13 +455,17 @@ module Groonga
         attr_writer :force_truncate
         attr_writer :force_lock_clear
 
-        def initialize
+        def initialize(output)
+          @output = output
           @context = Context.instance
         end
 
         def recover
           if @force_truncate
-            truncate_corrupt_objects
+            @database.each do |object|
+              next unless truncate_target?(object)
+              truncate_broken_object(object)
+            end
           end
           if @force_lock_clear
             clear_locks
@@ -465,27 +473,48 @@ module Groonga
           @database.recover
         end
 
-        def truncate_corrupt_objects
-          @database.each do |object|
-            next unless object.corrupt?
-            logger = @context.logger
-            object_path = object.path
-            object_dirname = File.dirname(object_path)
-            object_basename = File.basename(object_path)
-            object.truncate
-            Dir.foreach(object_dirname) do |path|
-              if path.start_with?("#{object_basename}.")
-                begin
-                  File.unlink("#{object_dirname}/#{path}")
-                  message = "Corrupted <#{object_path}> related file is removed: <#{path}>"
-                  $stdout.puts(message)
-                  logger.log(Logger::Level::INFO.to_i, __FILE__, __LINE__, "truncate_corrupt_objects", message)
-                rescue Error => error
-                  message = "Failed to remove file which is related to corrupted <#{object_path}>: <#{path}>"
-                  $stderr.puts(message)
-                  logger.log_error(message)
-                end
-              end
+        private
+        def truncate_target?(object)
+          return true if object.corrupt?
+
+          case object
+          when IndexColumn
+            # It'll be recovered later.
+            false
+          when Column, Table
+            object.locked?
+          else
+            false
+          end
+        end
+
+        def truncate_broken_object(object)
+          logger = @context.logger
+          name = object.name
+          object_path = object.path
+          object_dirname = File.dirname(object_path)
+          object_basename = File.basename(object_path)
+          object.truncate
+          message = "[#{name}] Truncated broken object: <#{object_path}>"
+          @output.puts(message)
+          logger.log(:info, message)
+
+          Dir.foreach(object_dirname) do |path|
+            next unless path.start_with?("#{object_basename}.")
+
+            full_path = "#{object_dirname}/#{path}"
+            begin
+              File.unlink(full_path)
+              message =
+                "[#{name}] Removed broken object related file: <#{full_path}>"
+              @output.puts(message)
+              logger.log(:info, message)
+            rescue Error => error
+              message =
+                "[#{name}] Failed to remove broken object related file: " +
+                "<#{full_path}>: #{error.class}: #{error.message}"
+              @output.puts(message)
+              logger.log_error(message)
             end
           end
         end
