@@ -1,6 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2015-2017 Brazil
+  Copyright(C) 2015-2018 Brazil
+  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -17,6 +18,7 @@
 */
 
 #include "grn.h"
+#include "grn_ctx_impl.h"
 #include "grn_index_column.h"
 #include "grn_pat.h"
 #include "grn_dat.h"
@@ -98,6 +100,26 @@ grn_obj_is_text_family_bulk(grn_ctx *ctx, grn_obj *obj)
   return GRN_TYPE_IS_TEXT_FAMILY(obj->header.domain);
 }
 
+grn_bool
+grn_obj_is_number_family_bulk(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_bulk(ctx, obj)) {
+    return GRN_FALSE;
+  }
+
+  return grn_type_id_is_number_family(ctx, obj->header.domain);
+}
+
+grn_bool
+grn_obj_is_vector(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!obj) {
+    return GRN_FALSE;
+  }
+
+  return obj->header.type == GRN_VECTOR;
+}
+
 grn_bool
 grn_obj_is_table(grn_ctx *ctx, grn_obj *obj)
 {
@@ -120,6 +142,27 @@ grn_obj_is_table(grn_ctx *ctx, grn_obj *obj)
   return is_table;
 }
 
+grn_bool
+grn_obj_is_lexicon(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_bool is_lexicon = GRN_FALSE;
+
+  if (!obj) {
+    return GRN_FALSE;
+  }
+
+  switch (obj->header.type) {
+  case GRN_TABLE_HASH_KEY :
+  case GRN_TABLE_PAT_KEY :
+  case GRN_TABLE_DAT_KEY :
+    is_lexicon = GRN_TRUE;
+  default :
+    break;
+  }
+
+  return is_lexicon;
+}
+
 grn_bool
 grn_obj_is_column(grn_ctx *ctx, grn_obj *obj)
 {
@@ -687,3 +730,91 @@ grn_obj_get_disk_usage(grn_ctx *ctx, grn_obj *obj)
 
   GRN_API_RETURN(usage);
 }
+
+grn_rc
+grn_obj_set_option_values(grn_ctx *ctx,
+                          grn_obj *obj,
+                          const char *name,
+                          int name_length,
+                          grn_obj *values)
+{
+  grn_id id;
+
+  GRN_API_ENTER;
+
+  id = grn_obj_id(ctx, obj);
+  if (id & GRN_OBJ_TMP_OBJECT) {
+    grn_options_set(ctx,
+                    ctx->impl->temporary_options,
+                    id & ~GRN_OBJ_TMP_OBJECT,
+                    name,
+                    name_length,
+                    values);
+  } else {
+    grn_db_set_option_values(ctx,
+                             grn_ctx_db(ctx),
+                             id,
+                             name,
+                             name_length,
+                             values);
+  }
+
+  GRN_API_RETURN(ctx->rc);
+}
+
+grn_option_revision
+grn_obj_get_option_values(grn_ctx *ctx,
+                          grn_obj *obj,
+                          const char *name,
+                          int name_length,
+                          grn_option_revision revision,
+                          grn_obj *values)
+{
+  grn_id id;
+  grn_option_revision returned_revision;
+
+  GRN_API_ENTER;
+
+  id = grn_obj_id(ctx, obj);
+  if (id & GRN_OBJ_TMP_OBJECT) {
+    returned_revision = grn_options_get(ctx,
+                                        ctx->impl->temporary_options,
+                                        id & ~GRN_OBJ_TMP_OBJECT,
+                                        name,
+                                        name_length,
+                                        revision,
+                                        values);
+  } else {
+    returned_revision = grn_db_get_option_values(ctx,
+                                                 grn_ctx_db(ctx),
+                                                 id,
+                                                 name,
+                                                 name_length,
+                                                 revision,
+                                                 values);
+  }
+
+  GRN_API_RETURN(returned_revision);
+}
+
+grn_rc
+grn_obj_clear_option_values(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_id id;
+  grn_rc rc;
+
+  GRN_API_ENTER;
+
+  id = grn_obj_id(ctx, obj);
+  if (id & GRN_OBJ_TMP_OBJECT) {
+    rc = grn_options_clear(ctx,
+                           ctx->impl->temporary_options,
+                           id & ~GRN_OBJ_TMP_OBJECT);
+  } else {
+    rc = grn_db_clear_option_values(ctx,
+                                    grn_ctx_db(ctx),
+                                    id);
+  }
+
+  GRN_API_RETURN(rc);
+}
