@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2012-2017 Brazil
+  Copyright(C) 2012-2018 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -15,8 +15,10 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
+
 #include "grn.h"
 #include "grn_ctx_impl_mrb.h"
+#include "grn_encoding.h"
 #include "grn_proc.h"
 #include <groonga/plugin.h>
 
@@ -47,6 +49,24 @@
 static grn_hash *grn_plugins = NULL;
 static grn_critical_section grn_plugins_lock;
 static grn_ctx grn_plugins_ctx;
+static grn_obj grn_plugins_path;
+
+#define GRN_PLUGINS_DIR_EACH_BEGIN(dir, dir_length) do {                \
+  unsigned int i_;                                                      \
+  const unsigned int n_dirs_ = grn_vector_size(&grn_plugins_ctx,        \
+                                               &grn_plugins_path);      \
+  for (i_ = 0; i_ < n_dirs_; i_++) {                                    \
+    const char *dir = NULL;                                             \
+    unsigned int dir_length;                                            \
+    dir_length = grn_vector_get_element(&grn_plugins_ctx,               \
+                                        &grn_plugins_path,              \
+                                        i_,                             \
+                                        &dir,                           \
+                                        NULL,                           \
+                                        NULL);
+#define GRN_PLUGINS_DIR_EACH_END()                                      \
+  }                                                                     \
+} while (GRN_FALSE)
 
 #ifdef HAVE_DLFCN_H
 #  include <dlfcn.h>
@@ -69,13 +89,17 @@ static grn_ctx grn_plugins_ctx;
 
 #define GRN_PLUGIN_KEY_SIZE(filename) (strlen((filename)) + 1)
 
-static char grn_plugins_dir[GRN_ENV_BUFFER_SIZE];
+static char grn_plugins_path_env[GRN_ENV_BUFFER_SIZE];
+static char grn_plugins_dir_env[GRN_ENV_BUFFER_SIZE];
 
 void
 grn_plugin_init_from_env(void)
 {
+  grn_getenv("GRN_PLUGINS_PATH",
+             grn_plugins_path_env,
+             GRN_ENV_BUFFER_SIZE);
   grn_getenv("GRN_PLUGINS_DIR",
-             grn_plugins_dir,
+             grn_plugins_dir_env,
              GRN_ENV_BUFFER_SIZE);
 }
 
@@ -116,8 +140,6 @@ grn_plugin_path(grn_ctx *ctx, grn_id id)
   const char *path;
   grn_plugin *plugin;
   int value_size;
-  const char *system_plugins_dir;
-  size_t system_plugins_dir_size;
 
   if (id == GRN_ID_NIL) {
     return NULL;
@@ -132,18 +154,18 @@ grn_plugin_path(grn_ctx *ctx, grn_id id)
   }
 
   path = plugin->path;
-  system_plugins_dir = grn_plugin_get_system_plugins_dir();
-  system_plugins_dir_size = strlen(system_plugins_dir);
-  if (strncmp(system_plugins_dir, path, system_plugins_dir_size) == 0) {
-    const char *plugin_name = path + system_plugins_dir_size;
-    while (plugin_name[0] == '/') {
-      plugin_name++;
+  GRN_PLUGINS_DIR_EACH_BEGIN(dir, dir_length) {
+    if (strncmp(dir, path, dir_length) == 0) {
+      const char *plugin_name = path + dir_length;
+      while (plugin_name[0] == '/') {
+        plugin_name++;
+      }
+      /* TODO: remove suffix too? */
+      return plugin_name;
     }
-    /* TODO: remove suffix too? */
-    return plugin_name;
-  } else {
-    return path;
-  }
+  } GRN_PLUGINS_DIR_EACH_END();
+
+  return path;
 }
 
 #define GRN_PLUGIN_FUNC_PREFIX "grn_plugin_impl_"
@@ -196,8 +218,20 @@ grn_plugin_call_register_mrb(grn_ctx *ctx, grn_id id, grn_plugin *plugin)
 
   arena_index = mrb_gc_arena_save(mrb);
   plugin_loader_class = mrb_class_get_under(mrb, module, "PluginLoader");
-  mrb_funcall(mrb, mrb_obj_value(plugin_loader_class),
-              "load_file", 1, mrb_str_new_cstr(mrb, ctx->impl->plugin_path));
+  {
+    const char *utf8_path;
+    mrb_value mrb_path;
+    utf8_path =
+      grn_encoding_convert_to_utf8_from_locale(ctx,
+                                               ctx->impl->plugin_path,
+                                               -1,
+                                               NULL);
+    mrb_path = mrb_str_new_cstr(mrb, utf8_path);
+    grn_encoding_converted_free(ctx, utf8_path);
+    mrb_funcall(mrb, mrb_obj_value(plugin_loader_class),
+                "load_file",
+                1, mrb_path);
+  }
   mrb_gc_arena_restore(mrb, arena_index);
   return ctx->rc;
 }
@@ -356,8 +390,8 @@ grn_plugin_open(grn_ctx *ctx, const char *filename)
     const char *mrb_suffix;
     mrb_suffix = grn_plugin_get_ruby_suffix();
     if (filename_size > strlen(mrb_suffix) &&
-      strcmp(filename + (strlen(filename) - strlen(mrb_suffix)),
-             mrb_suffix) == 0) {
+        strcmp(filename + (strlen(filename) - strlen(mrb_suffix)),
+               mrb_suffix) == 0) {
       id = grn_plugin_open_mrb(ctx, filename, filename_size);
       goto exit;
     }
@@ -483,18 +517,112 @@ exit:
   return func;
 }
 
+#ifdef WIN32
+static void
+grn_plugins_init_path(grn_ctx *ctx, grn_obj *path, const char *path_env)
+{
+  /* TODO */
+}
+#else /* WIN32 */
+static void
+grn_plugins_init_path(grn_ctx *ctx, grn_obj *path, const char *path_env)
+{
+  const char separator = ':';
+  const char *start = path_env;
+  const char *current;
+
+  for (current = path_env; current[0]; current++) {
+    if (current[0] == separator) {
+      if (current - start > 0) {
+        grn_vector_add_element(&grn_plugins_ctx,
+                               &grn_plugins_path,
+                               start,
+                               current - start,
+                               0,
+                               GRN_DB_TEXT);
+      }
+      start = current + 1;
+    }
+  }
+
+  if (start[0] && current - start > 0) {
+    grn_vector_add_element(&grn_plugins_ctx,
+                           &grn_plugins_path,
+                           start,
+                           current - start,
+                           0,
+                           GRN_DB_TEXT);
+  }
+}
+#endif /* WIN32 */
+
+#ifdef WIN32
+static char *windows_plugins_dir = NULL;
+static char windows_plugins_dir_buffer[PATH_MAX];
+static const char *
+grn_plugin_get_default_system_plugins_dir(void)
+{
+  if (!windows_plugins_dir) {
+    const char *base_dir;
+    const char *relative_path = GRN_RELATIVE_PLUGINS_DIR;
+    size_t base_dir_length;
+
+    base_dir = grn_windows_base_dir();
+    base_dir_length = strlen(base_dir);
+    grn_strcpy(windows_plugins_dir_buffer, PATH_MAX, base_dir);
+    grn_strcat(windows_plugins_dir_buffer, PATH_MAX, "/");
+    grn_strcat(windows_plugins_dir_buffer, PATH_MAX, relative_path);
+    windows_plugins_dir = windows_plugins_dir_buffer;
+  }
+  return windows_plugins_dir;
+}
+
+#else /* WIN32 */
+static const char *
+grn_plugin_get_default_system_plugins_dir(void)
+{
+  return GRN_PLUGINS_DIR;
+}
+#endif /* WIN32 */
+
+const char *
+grn_plugin_get_system_plugins_dir(void)
+{
+  if (grn_plugins_dir_env[0]) {
+    return grn_plugins_dir_env;
+  } else {
+    return grn_plugin_get_default_system_plugins_dir();
+  }
+}
+
 grn_rc
 grn_plugins_init(void)
 {
   CRITICAL_SECTION_INIT(grn_plugins_lock);
   grn_ctx_init(&grn_plugins_ctx, 0);
+  GRN_TEXT_INIT(&grn_plugins_path, GRN_OBJ_VECTOR);
   grn_plugins = grn_hash_create(&grn_plugins_ctx, NULL,
                                 PATH_MAX, sizeof(grn_plugin *),
                                 GRN_OBJ_KEY_VAR_SIZE);
   if (!grn_plugins) {
+    GRN_OBJ_FIN(&grn_plugins_ctx, &grn_plugins_path);
     grn_ctx_fin(&grn_plugins_ctx);
     return GRN_NO_MEMORY_AVAILABLE;
   }
+  if (grn_plugins_path_env[0]) {
+    grn_plugins_init_path(&grn_plugins_ctx,
+                          &grn_plugins_path,
+                          grn_plugins_path_env);
+  }
+  {
+    const char *system_plugins_dir = grn_plugin_get_system_plugins_dir();
+    grn_vector_add_element(&grn_plugins_ctx,
+                           &grn_plugins_path,
+                           system_plugins_dir,
+                           strlen(system_plugins_dir),
+                           0,
+                           GRN_DB_TEXT);
+  }
   return GRN_SUCCESS;
 }
 
@@ -507,6 +635,7 @@ grn_plugins_fin(void)
     grn_plugin_close(&grn_plugins_ctx, id);
   });
   rc = grn_hash_close(&grn_plugins_ctx, grn_plugins);
+  GRN_OBJ_FIN(&grn_plugins_ctx, &grn_plugins_path);
   grn_ctx_fin(&grn_plugins_ctx);
   CRITICAL_SECTION_FIN(grn_plugins_lock);
   return rc;
@@ -548,45 +677,6 @@ grn_plugin_register_by_path(grn_ctx *ctx, const char *path)
   GRN_API_RETURN(ctx->rc);
 }
 
-#ifdef WIN32
-static char *windows_plugins_dir = NULL;
-static char windows_plugins_dir_buffer[PATH_MAX];
-static const char *
-grn_plugin_get_default_system_plugins_dir(void)
-{
-  if (!windows_plugins_dir) {
-    const char *base_dir;
-    const char *relative_path = GRN_RELATIVE_PLUGINS_DIR;
-    size_t base_dir_length;
-
-    base_dir = grn_windows_base_dir();
-    base_dir_length = strlen(base_dir);
-    grn_strcpy(windows_plugins_dir_buffer, PATH_MAX, base_dir);
-    grn_strcat(windows_plugins_dir_buffer, PATH_MAX, "/");
-    grn_strcat(windows_plugins_dir_buffer, PATH_MAX, relative_path);
-    windows_plugins_dir = windows_plugins_dir_buffer;
-  }
-  return windows_plugins_dir;
-}
-
-#else /* WIN32 */
-static const char *
-grn_plugin_get_default_system_plugins_dir(void)
-{
-  return GRN_PLUGINS_DIR;
-}
-#endif /* WIN32 */
-
-const char *
-grn_plugin_get_system_plugins_dir(void)
-{
-  if (grn_plugins_dir[0]) {
-    return grn_plugins_dir;
-  } else {
-    return grn_plugin_get_default_system_plugins_dir();
-  }
-}
-
 static char *
 grn_plugin_find_path_raw(grn_ctx *ctx, const char *path)
 {
@@ -697,40 +787,12 @@ grn_plugin_find_path_libs_so(grn_ctx *ctx, const char *path, size_t path_len)
   return grn_plugin_find_path_raw(ctx, libs_so_path);
 }
 
-char *
-grn_plugin_find_path(grn_ctx *ctx, const char *name)
+static char *
+grn_plugin_find_path_one(grn_ctx *ctx, const char *path)
 {
-  const char *plugins_dir;
-  char dir_last_char;
-  char path[PATH_MAX];
-  int name_length, max_name_length;
   char *found_path = NULL;
   size_t path_len;
 
-  GRN_API_ENTER;
-  if (name[0] == '/') {
-    path[0] = '\0';
-  } else {
-    plugins_dir = grn_plugin_get_system_plugins_dir();
-    grn_strcpy(path, PATH_MAX, plugins_dir);
-
-    dir_last_char = plugins_dir[strlen(path) - 1];
-    if (dir_last_char != '/') {
-      grn_strcat(path, PATH_MAX, "/");
-    }
-  }
-
-  name_length = strlen(name);
-  max_name_length = PATH_MAX - strlen(path) - 1;
-  if (name_length > max_name_length) {
-    ERR(GRN_INVALID_ARGUMENT,
-        "plugin name is too long: %d (max: %d) <%s%s>",
-        name_length, max_name_length,
-        path, name);
-    goto exit;
-  }
-  grn_strcat(path, PATH_MAX, name);
-
   found_path = grn_plugin_find_path_raw(ctx, path);
   if (found_path) {
     goto exit;
@@ -763,6 +825,53 @@ grn_plugin_find_path(grn_ctx *ctx, const char *name)
   }
 
 exit :
+  return found_path;
+}
+
+char *
+grn_plugin_find_path(grn_ctx *ctx, const char *name)
+{
+  char *found_path = NULL;
+
+  GRN_API_ENTER;
+
+  if (name[0] == '/') {
+    found_path = grn_plugin_find_path_one(ctx, name);
+  } else {
+    GRN_PLUGINS_DIR_EACH_BEGIN(dir, dir_length) {
+      char dir_last_char;
+      char path[PATH_MAX];
+      int name_length, max_name_length;
+
+      grn_strncpy(path, PATH_MAX, dir, dir_length);
+      path[dir_length] = '\0';
+
+      dir_last_char = dir[dir_length - 1];
+      if (dir_last_char != '/') {
+        grn_strcat(path, PATH_MAX, "/");
+      }
+
+      name_length = strlen(name);
+      max_name_length = PATH_MAX - strlen(path) - 1;
+      if (name_length > max_name_length) {
+        ERR(GRN_INVALID_ARGUMENT,
+            "plugin name is too long: %d (max: %d) <%s%s>",
+            name_length, max_name_length,
+            path, name);
+        break;
+      }
+      grn_strcat(path, PATH_MAX, name);
+
+      found_path = grn_plugin_find_path_one(ctx, path);
+      if (found_path) {
+        break;
+      }
+      if (ctx->rc != GRN_SUCCESS) {
+        break;
+      }
+    } GRN_PLUGINS_DIR_EACH_END();
+  }
+
   GRN_API_RETURN(found_path);
 }
 
@@ -770,24 +879,34 @@ static void
 grn_plugin_set_name_resolve_error(grn_ctx *ctx, const char *name,
                                   const char *tag)
 {
-  const char *prefix, *prefix_separator, *suffix;
-
   if (name[0] == '/') {
-    prefix = "";
-    prefix_separator = "";
-    suffix = "";
+    ERR(GRN_NO_SUCH_FILE_OR_DIRECTORY,
+        "%s cannot find plugin file: <%s>",
+        tag, name);
   } else {
-    prefix = grn_plugin_get_system_plugins_dir();
-    if (prefix[strlen(prefix) - 1] != '/') {
-      prefix_separator = "/";
+    const char *grn_encoding_plugins_dir;
+    size_t grn_encoding_plugins_dir_length;
+    const char *plugins_dir_path_separator;
+
+    grn_encoding_plugins_dir =
+      grn_encoding_convert_from_locale(ctx,
+                                       grn_plugin_get_system_plugins_dir(),
+                                       -1,
+                                       &grn_encoding_plugins_dir_length);
+    if (grn_encoding_plugins_dir[grn_encoding_plugins_dir_length - 1] != '/') {
+      plugins_dir_path_separator = "/";
     } else {
-      prefix_separator = "";
+      plugins_dir_path_separator = "";
     }
-    suffix = grn_plugin_get_suffix();
+    ERR(GRN_NO_SUCH_FILE_OR_DIRECTORY,
+        "%s cannot find plugin file: <%s%s%s%s>",
+        tag,
+        grn_encoding_plugins_dir,
+        plugins_dir_path_separator,
+        name,
+        grn_plugin_get_suffix());
+    grn_encoding_converted_free(ctx, grn_encoding_plugins_dir);
   }
-  ERR(GRN_NO_SUCH_FILE_OR_DIRECTORY,
-      "%s cannot find plugin file: <%s%s%s%s>",
-      tag, prefix, prefix_separator, name, suffix);
 }
 
 grn_rc
@@ -1404,3 +1523,12 @@ grn_plugin_proc_get_value_int64(grn_ctx *ctx,
 {
   return grn_proc_get_value_int64(ctx, value, default_value_raw, context);
 }
+
+grn_operator
+grn_plugin_proc_get_value_mode(grn_ctx *ctx,
+                               grn_obj *value,
+                               grn_operator default_mode,
+                               const char *context)
+{
+  return grn_proc_get_value_mode(ctx, value, default_mode, context);
+}
