@@ -1,6 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2009-2016 Brazil
+  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,13 +25,13 @@
 
 #include <groonga/plugin.h>
 
-grn_obj_flags
+grn_column_flags
 grn_proc_column_parse_flags(grn_ctx *ctx,
                             const char *error_message_tag,
                             const char *text,
                             const char *end)
 {
-  grn_obj_flags flags = 0;
+  grn_column_flags flags = 0;
   while (text < end) {
     size_t name_size;
 
@@ -53,10 +54,13 @@ grn_proc_column_parse_flags(grn_ctx *ctx,
     CHECK_FLAG(COLUMN_INDEX);
     CHECK_FLAG(COMPRESS_ZLIB);
     CHECK_FLAG(COMPRESS_LZ4);
+    CHECK_FLAG(COMPRESS_ZSTD);
     CHECK_FLAG(WITH_SECTION);
     CHECK_FLAG(WITH_WEIGHT);
     CHECK_FLAG(WITH_POSITION);
     CHECK_FLAG(RING_BUFFER);
+    CHECK_FLAG(INDEX_SMALL);
+    CHECK_FLAG(INDEX_MEDIUM);
 
 #undef CHECK_FLAG
 
@@ -175,7 +179,7 @@ command_column_create(grn_ctx *ctx, int nargs, grn_obj **args,
   grn_obj *flags_raw;
   grn_obj *type_raw;
   grn_obj *source_raw;
-  grn_obj_flags flags;
+  grn_column_flags flags;
   grn_obj *type = NULL;
 
   table_raw  = grn_plugin_proc_get_var(ctx, user_data, "table", -1);
@@ -320,11 +324,23 @@ command_column_remove(grn_ctx *ctx, int nargs, grn_obj **args,
   }
 
   fullname[fullname_len] = GRN_DB_DELIMITER;
-  grn_memcpy((fullname + fullname_len + 1),
+  fullname_len++;
+  if (fullname_len + GRN_TEXT_LEN(name) > GRN_TABLE_MAX_KEY_SIZE) {
+    GRN_PLUGIN_ERROR(ctx,
+                     GRN_INVALID_ARGUMENT,
+                     "[column][remove] column name is too long: <%d> > <%u>: "
+                     "<%.*s>",
+                     (int)GRN_TEXT_LEN(name),
+                     GRN_TABLE_MAX_KEY_SIZE - fullname_len,
+                     (int)GRN_TEXT_LEN(name),
+                     GRN_TEXT_VALUE(name));
+    grn_ctx_output_bool(ctx, GRN_FALSE);
+    return NULL;
+  }
+  grn_memcpy(fullname + fullname_len,
              GRN_TEXT_VALUE(name),
              GRN_TEXT_LEN(name));
-  fullname_len += GRN_TEXT_LEN(name) + 1;
-  /* TODO:check fullname_len < GRN_TABLE_MAX_KEY_SIZE */
+  fullname_len += GRN_TEXT_LEN(name);
   column = grn_ctx_get(ctx, fullname, fullname_len);
   if (!column) {
     GRN_PLUGIN_ERROR(ctx,
@@ -520,7 +536,7 @@ output_column_info(grn_ctx *ctx, grn_obj *column)
   output_column_name(ctx, column);
   grn_ctx_output_cstr(ctx, path);
   grn_ctx_output_cstr(ctx, type);
-  grn_dump_column_create_flags(ctx, column->header.flags, &o);
+  grn_dump_column_create_flags(ctx, grn_column_get_flags(ctx, column), &o);
   grn_ctx_output_obj(ctx, &o, NULL);
   grn_proc_output_object_id_name(ctx, column->header.domain);
   grn_proc_output_object_id_name(ctx, grn_obj_get_range(ctx, column));
@@ -736,167 +752,6 @@ command_column_copy_resolve_target(grn_ctx *ctx,
   return ctx->rc;
 }
 
-static void
-command_column_copy_same_table(grn_ctx *ctx, grn_obj *table,
-                               grn_obj *from_column, grn_obj *to_column)
-{
-  grn_table_cursor *cursor;
-  grn_id id;
-  grn_obj value;
-
-  cursor = grn_table_cursor_open(ctx, table,
-                                 NULL, 0,
-                                 NULL, 0,
-                                 0, -1, 0);
-  if (!cursor) {
-    return;
-  }
-
-  GRN_VOID_INIT(&value);
-  while ((id = grn_table_cursor_next(ctx, cursor)) != GRN_ID_NIL) {
-    GRN_BULK_REWIND(&value);
-    grn_obj_get_value(ctx, from_column, id, &value);
-    grn_obj_set_value(ctx, to_column, id, &value, GRN_OBJ_SET);
-  }
-  GRN_OBJ_FIN(ctx, &value);
-  grn_table_cursor_close(ctx, cursor);
-}
-
-static void
-command_column_copy_same_key_type(grn_ctx *ctx,
-                                  grn_obj *from_table,
-                                  grn_obj *from_column,
-                                  grn_obj *to_table,
-                                  grn_obj *to_column)
-{
-  grn_table_cursor *cursor;
-  grn_id from_id;
-  grn_obj value;
-
-  cursor = grn_table_cursor_open(ctx, from_table,
-                                 NULL, 0,
-                                 NULL, 0,
-                                 0, -1, 0);
-  if (!cursor) {
-    return;
-  }
-
-  GRN_VOID_INIT(&value);
-  while ((from_id = grn_table_cursor_next(ctx, cursor)) != GRN_ID_NIL) {
-    void *key;
-    int key_size;
-    grn_id to_id;
-
-    key_size = grn_table_cursor_get_key(ctx, cursor, &key);
-    to_id = grn_table_add(ctx, to_table, key, key_size, NULL);
-    if (to_id == GRN_ID_NIL) {
-      continue;
-    }
-
-    GRN_BULK_REWIND(&value);
-    grn_obj_get_value(ctx, from_column, from_id, &value);
-    grn_obj_set_value(ctx, to_column, to_id, &value, GRN_OBJ_SET);
-  }
-  GRN_OBJ_FIN(ctx, &value);
-  grn_table_cursor_close(ctx, cursor);
-}
-
-static void
-command_column_copy_different(grn_ctx *ctx,
-                              grn_obj *from_table,
-                              grn_obj *from_column,
-                              grn_obj *to_table,
-                              grn_obj *to_column,
-                              grn_obj *from_table_name,
-                              grn_obj *from_column_name,
-                              grn_obj *to_table_name,
-                              grn_obj *to_column_name)
-{
-  grn_table_cursor *cursor;
-  grn_id from_id;
-  grn_obj from_key_buffer;
-  grn_obj to_key_buffer;
-  grn_obj value;
-
-  cursor = grn_table_cursor_open(ctx, from_table,
-                                 NULL, 0,
-                                 NULL, 0,
-                                 0, -1, 0);
-  if (!cursor) {
-    return;
-  }
-
-  if (from_table->header.domain == GRN_DB_SHORT_TEXT) {
-    GRN_SHORT_TEXT_INIT(&from_key_buffer, 0);
-  } else {
-    GRN_VALUE_FIX_SIZE_INIT(&from_key_buffer, 0, from_table->header.domain);
-  }
-  if (to_table->header.domain == GRN_DB_SHORT_TEXT) {
-    GRN_SHORT_TEXT_INIT(&to_key_buffer, 0);
-  } else {
-    GRN_VALUE_FIX_SIZE_INIT(&to_key_buffer, 0, to_table->header.domain);
-  }
-  GRN_VOID_INIT(&value);
-  while ((from_id = grn_table_cursor_next(ctx, cursor)) != GRN_ID_NIL) {
-    void *key;
-    int key_size;
-    grn_rc cast_rc;
-    grn_id to_id;
-
-    GRN_BULK_REWIND(&from_key_buffer);
-    GRN_BULK_REWIND(&to_key_buffer);
-
-    key_size = grn_table_cursor_get_key(ctx, cursor, &key);
-    grn_bulk_write(ctx, &from_key_buffer, key, key_size);
-    cast_rc = grn_obj_cast(ctx, &from_key_buffer, &to_key_buffer, GRN_FALSE);
-    if (cast_rc != GRN_SUCCESS) {
-      grn_obj *to_key_type;
-      grn_obj inspected_key;
-      grn_obj inspected_to_key_type;
-
-      to_key_type = grn_ctx_at(ctx, to_table->header.domain);
-      GRN_TEXT_INIT(&inspected_key, 0);
-      GRN_TEXT_INIT(&inspected_to_key_type, 0);
-      grn_inspect(ctx, &inspected_key, &from_key_buffer);
-      grn_inspect(ctx, &inspected_to_key_type, to_key_type);
-      ERR(cast_rc,
-          "[column][copy] failed to cast key: <%.*s> -> %.*s: "
-          "<%.*s.%.*s> -> <%.*s.%.*s>",
-          (int)GRN_TEXT_LEN(&inspected_key),
-          GRN_TEXT_VALUE(&inspected_key),
-          (int)GRN_TEXT_LEN(&inspected_to_key_type),
-          GRN_TEXT_VALUE(&inspected_to_key_type),
-          (int)GRN_TEXT_LEN(from_table_name),
-          GRN_TEXT_VALUE(from_table_name),
-          (int)GRN_TEXT_LEN(from_column_name),
-          GRN_TEXT_VALUE(from_column_name),
-          (int)GRN_TEXT_LEN(to_table_name),
-          GRN_TEXT_VALUE(to_table_name),
-          (int)GRN_TEXT_LEN(to_column_name),
-          GRN_TEXT_VALUE(to_column_name));
-      GRN_OBJ_FIN(ctx, &inspected_key);
-      GRN_OBJ_FIN(ctx, &inspected_to_key_type);
-      break;
-    }
-    to_id = grn_table_add(ctx, to_table,
-                          GRN_BULK_HEAD(&to_key_buffer),
-                          GRN_BULK_VSIZE(&to_key_buffer),
-                          NULL);
-    if (to_id == GRN_ID_NIL) {
-      continue;
-    }
-
-    GRN_BULK_REWIND(&value);
-    grn_obj_get_value(ctx, from_column, from_id, &value);
-    grn_obj_set_value(ctx, to_column, to_id, &value, GRN_OBJ_SET);
-  }
-  GRN_OBJ_FIN(ctx, &from_key_buffer);
-  GRN_OBJ_FIN(ctx, &to_key_buffer);
-  GRN_OBJ_FIN(ctx, &value);
-
-  grn_table_cursor_close(ctx, cursor);
-}
-
 static grn_obj *
 command_column_copy(grn_ctx *ctx, int nargs, grn_obj **args,
                     grn_user_data *user_data)
@@ -929,44 +784,7 @@ command_column_copy(grn_ctx *ctx, int nargs, grn_obj **args,
     goto exit;
   }
 
-  if ((from_table->header.type == GRN_TABLE_NO_KEY ||
-       to_table->header.type == GRN_TABLE_NO_KEY) &&
-      from_table != to_table) {
-    rc = GRN_OPERATION_NOT_SUPPORTED;
-    GRN_PLUGIN_ERROR(ctx,
-                     rc,
-                     "[column][copy] copy from/to TABLE_NO_KEY isn't supported: "
-                     "<%.*s%c%.*s> -> <%.*s%c%.*s>",
-                     (int)GRN_TEXT_LEN(from_table_name),
-                     GRN_TEXT_VALUE(from_table_name),
-                     GRN_DB_DELIMITER,
-                     (int)GRN_TEXT_LEN(from_column_name),
-                     GRN_TEXT_VALUE(from_column_name),
-                     (int)GRN_TEXT_LEN(to_table_name),
-                     GRN_TEXT_VALUE(to_table_name),
-                     GRN_DB_DELIMITER,
-                     (int)GRN_TEXT_LEN(to_column_name),
-                     GRN_TEXT_VALUE(to_column_name));
-    goto exit;
-  }
-
-  if (from_table == to_table) {
-    command_column_copy_same_table(ctx, from_table, from_column, to_column);
-  } else if (from_table->header.domain == to_table->header.domain) {
-    command_column_copy_same_key_type(ctx,
-                                      from_table, from_column,
-                                      to_table, to_column);
-  } else {
-    command_column_copy_different(ctx,
-                                  from_table,
-                                  from_column,
-                                  to_table,
-                                  to_column,
-                                  from_table_name,
-                                  from_column_name,
-                                  to_table_name,
-                                  to_column_name);
-  }
+  rc = grn_column_copy(ctx, from_column, to_column);
 
 exit :
   grn_ctx_output_bool(ctx, rc == GRN_SUCCESS);
