@@ -1,6 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2016 Brazil
+  Copyright(C) 2009-2018 Brazil
+  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -30,6 +31,8 @@ typedef struct {
   grn_bool is_close_opened_object_mode;
   grn_bool have_reference_column;
   grn_bool have_index_column;
+  grn_bool is_sort_hash_table;
+  grn_obj column_name_buffer;
 } grn_dumper;
 
 static void
@@ -391,37 +394,119 @@ dump_record_column_vector(grn_ctx *ctx, grn_dumper *dumper, grn_id id,
                           grn_obj *column, grn_id range_id, grn_obj *buf)
 {
   grn_obj *range;
+  grn_obj_format *format_argument = NULL;
+  grn_obj_format format;
 
   range = grn_ctx_at(ctx, range_id);
+  if (column->header.flags & GRN_OBJ_WITH_WEIGHT) {
+    format.flags = GRN_OBJ_FORMAT_WITH_WEIGHT;
+    format_argument = &format;
+  }
+
   if (grn_obj_is_table(ctx, range) ||
       (range->header.flags & GRN_OBJ_KEY_VAR_SIZE) == 0) {
     GRN_OBJ_INIT(buf, GRN_UVECTOR, 0, range_id);
     grn_obj_get_value(ctx, column, id, buf);
-    grn_text_otoj(ctx, dumper->output, buf, NULL);
+    grn_text_otoj(ctx, dumper->output, buf, format_argument);
   } else {
-    grn_obj_format *format_argument = NULL;
-    grn_obj_format format;
-    if (column->header.flags & GRN_OBJ_WITH_WEIGHT) {
-      format.flags = GRN_OBJ_FORMAT_WITH_WEIGHT;
-      format_argument = &format;
-    }
     GRN_OBJ_INIT(buf, GRN_VECTOR, 0, range_id);
     grn_obj_get_value(ctx, column, id, buf);
     grn_text_otoj(ctx, dumper->output, buf, format_argument);
   }
+
   grn_obj_unlink(ctx, range);
   grn_obj_unlink(ctx, buf);
 }
 
+static void
+dump_record(grn_ctx *ctx, grn_dumper *dumper,
+            grn_obj *table,
+            grn_id id,
+            grn_obj *columns, int n_columns)
+{
+  int j;
+  grn_obj buf;
+  grn_obj *column_name = &(dumper->column_name_buffer);
+
+  GRN_TEXT_PUTC(ctx, dumper->output, '[');
+  for (j = 0; j < n_columns; j++) {
+    grn_bool is_value_column;
+    grn_id range;
+    grn_obj *column;
+    column = GRN_PTR_VALUE_AT(columns, j);
+    /* TODO: use grn_obj_is_value_accessor() */
+    GRN_BULK_REWIND(column_name);
+    grn_column_name_(ctx, column, column_name);
+    if (GRN_TEXT_LEN(column_name) == GRN_COLUMN_NAME_VALUE_LEN &&
+        !memcmp(GRN_TEXT_VALUE(column_name),
+                GRN_COLUMN_NAME_VALUE,
+                GRN_COLUMN_NAME_VALUE_LEN)) {
+      is_value_column = GRN_TRUE;
+    } else {
+      is_value_column = GRN_FALSE;
+    }
+    range = grn_obj_get_range(ctx, column);
+
+    if (j) { GRN_TEXT_PUTC(ctx, dumper->output, ','); }
+    switch (column->header.type) {
+    case GRN_COLUMN_VAR_SIZE:
+    case GRN_COLUMN_FIX_SIZE:
+      switch (column->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) {
+      case GRN_OBJ_COLUMN_VECTOR:
+        dump_record_column_vector(ctx, dumper, id, column, range, &buf);
+        break;
+      case GRN_OBJ_COLUMN_SCALAR:
+        {
+          GRN_OBJ_INIT(&buf, GRN_BULK, 0, range);
+          grn_obj_get_value(ctx, column, id, &buf);
+          grn_text_otoj(ctx, dumper->output, &buf, NULL);
+          grn_obj_unlink(ctx, &buf);
+        }
+        break;
+      default:
+        GRN_PLUGIN_ERROR(ctx,
+                         GRN_OPERATION_NOT_SUPPORTED,
+                         "unsupported column type: %#x",
+                         column->header.type);
+        break;
+      }
+      break;
+    case GRN_COLUMN_INDEX:
+      break;
+    case GRN_ACCESSOR:
+      {
+        GRN_OBJ_INIT(&buf, GRN_BULK, 0, range);
+        grn_obj_get_value(ctx, column, id, &buf);
+        /* XXX maybe, grn_obj_get_range() should not unconditionally return
+           GRN_DB_INT32 when column is GRN_ACCESSOR and
+           GRN_ACCESSOR_GET_VALUE */
+        if (is_value_column) {
+          buf.header.domain = grn_obj_get_range(ctx, table);
+        }
+        grn_text_otoj(ctx, dumper->output, &buf, NULL);
+        grn_obj_unlink(ctx, &buf);
+      }
+      break;
+    default:
+      GRN_PLUGIN_ERROR(ctx,
+                       GRN_OPERATION_NOT_SUPPORTED,
+                       "unsupported header type %#x",
+                       column->header.type);
+      break;
+    }
+  }
+  GRN_TEXT_PUTC(ctx, dumper->output, ']');
+  if (GRN_TEXT_LEN(dumper->output) >= DUMP_FLUSH_THRESHOLD_SIZE) {
+    grn_ctx_output_flush(ctx, 0);
+  }
+}
+
 static void
 dump_records(grn_ctx *ctx, grn_dumper *dumper, grn_obj *table)
 {
-  grn_id old_id = 0, id;
   grn_table_cursor *cursor;
   int i, n_columns;
-  grn_obj column_name;
   grn_obj columns;
-  grn_obj delete_commands;
   grn_bool have_index_column = GRN_FALSE;
   grn_bool have_data_column = GRN_FALSE;
 
@@ -433,7 +518,6 @@ dump_records(grn_ctx *ctx, grn_dumper *dumper, grn_obj *table)
     grn_ctx_push_temporary_open_space(ctx);
   }
 
-  GRN_TEXT_INIT(&column_name, 0);
   GRN_PTR_INIT(&columns, GRN_OBJ_VECTOR, GRN_ID_NIL);
 
   if (table->header.type == GRN_TABLE_NO_KEY) {
@@ -520,114 +604,91 @@ dump_records(grn_ctx *ctx, grn_dumper *dumper, grn_obj *table)
   GRN_TEXT_PUTC(ctx, dumper->output, '[');
   for (i = 0; i < n_columns; i++) {
     grn_obj *column;
+    grn_obj *column_name = &(dumper->column_name_buffer);
+
     column = GRN_PTR_VALUE_AT(&columns, i);
     if (i) { GRN_TEXT_PUTC(ctx, dumper->output, ','); }
-    GRN_BULK_REWIND(&column_name);
-    grn_column_name_(ctx, column, &column_name);
-    grn_text_otoj(ctx, dumper->output, &column_name, NULL);
+    GRN_BULK_REWIND(column_name);
+    grn_column_name_(ctx, column, column_name);
+    grn_text_otoj(ctx, dumper->output, column_name, NULL);
   }
   GRN_TEXT_PUTS(ctx, dumper->output, "],\n");
 
-  GRN_TEXT_INIT(&delete_commands, 0);
-  cursor = grn_table_cursor_open(ctx, table, NULL, 0, NULL, 0, 0, -1,
-                                 GRN_CURSOR_BY_KEY);
-  for (i = 0; (id = grn_table_cursor_next(ctx, cursor)) != GRN_ID_NIL;
-       ++i, old_id = id) {
-    int j;
-    grn_obj buf;
-    if (i) { GRN_TEXT_PUTS(ctx, dumper->output, ",\n"); }
-    if (table->header.type == GRN_TABLE_NO_KEY && old_id + 1 < id) {
-      grn_id current_id;
-      for (current_id = old_id + 1; current_id < id; current_id++) {
-        GRN_TEXT_PUTS(ctx, dumper->output, "[],\n");
-        GRN_TEXT_PUTS(ctx, &delete_commands, "delete --table ");
-        dump_obj_name_raw(ctx, &delete_commands, table);
-        GRN_TEXT_PUTS(ctx, &delete_commands, " --id ");
-        grn_text_lltoa(ctx, &delete_commands, current_id);
-        GRN_TEXT_PUTC(ctx, &delete_commands, '\n');
-      }
-    }
-    GRN_TEXT_PUTC(ctx, dumper->output, '[');
-    for (j = 0; j < n_columns; j++) {
-      grn_bool is_value_column;
-      grn_id range;
-      grn_obj *column;
-      column = GRN_PTR_VALUE_AT(&columns, j);
-      /* TODO: use grn_obj_is_value_accessor() */
-      GRN_BULK_REWIND(&column_name);
-      grn_column_name_(ctx, column, &column_name);
-      if (GRN_TEXT_LEN(&column_name) == GRN_COLUMN_NAME_VALUE_LEN &&
-          !memcmp(GRN_TEXT_VALUE(&column_name),
-                  GRN_COLUMN_NAME_VALUE,
-                  GRN_COLUMN_NAME_VALUE_LEN)) {
-        is_value_column = GRN_TRUE;
+  if (table->header.type == GRN_TABLE_HASH_KEY && dumper->is_sort_hash_table) {
+    grn_obj *sorted;
+    grn_table_sort_key sort_keys[1];
+    uint32_t n_sort_keys = 1;
+    grn_bool is_first_record = GRN_TRUE;
+
+    sort_keys[0].key = grn_obj_column(ctx, table,
+                                      GRN_COLUMN_NAME_KEY,
+                                      GRN_COLUMN_NAME_KEY_LEN);
+    sort_keys[0].flags = GRN_TABLE_SORT_ASC;
+    sort_keys[0].offset = 0;
+    sorted = grn_table_create(ctx,
+                              NULL, 0, NULL,
+                              GRN_TABLE_NO_KEY,
+                              NULL,
+                              table);
+    grn_table_sort(ctx,
+                   table, 0, -1,
+                   sorted,
+                   sort_keys, n_sort_keys);
+    cursor = grn_table_cursor_open(ctx,
+                                   sorted,
+                                   NULL, 0, NULL, 0,
+                                   0, -1,
+                                   0);
+    while (grn_table_cursor_next(ctx, cursor) != GRN_ID_NIL) {
+      void *value_raw;
+      grn_id id;
+
+      grn_table_cursor_get_value(ctx, cursor, &value_raw);
+      id = *((grn_id *)value_raw);
+
+      if (is_first_record) {
+        is_first_record = GRN_FALSE;
       } else {
-        is_value_column = GRN_FALSE;
+        GRN_TEXT_PUTS(ctx, dumper->output, ",\n");
       }
-      range = grn_obj_get_range(ctx, column);
+      dump_record(ctx, dumper, table, id, &columns, n_columns);
+    }
+    GRN_TEXT_PUTS(ctx, dumper->output, "\n]\n");
+    grn_obj_close(ctx, sorted);
+    grn_obj_unlink(ctx, sort_keys[0].key);
+  } else {
+    grn_obj delete_commands;
+    grn_id old_id = GRN_ID_NIL;
+    grn_id id;
 
-      if (j) { GRN_TEXT_PUTC(ctx, dumper->output, ','); }
-      switch (column->header.type) {
-      case GRN_COLUMN_VAR_SIZE:
-      case GRN_COLUMN_FIX_SIZE:
-        switch (column->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) {
-        case GRN_OBJ_COLUMN_VECTOR:
-          dump_record_column_vector(ctx, dumper, id, column, range, &buf);
-          break;
-        case GRN_OBJ_COLUMN_SCALAR:
-          {
-            GRN_OBJ_INIT(&buf, GRN_BULK, 0, range);
-            grn_obj_get_value(ctx, column, id, &buf);
-            grn_text_otoj(ctx, dumper->output, &buf, NULL);
-            grn_obj_unlink(ctx, &buf);
-          }
-          break;
-        default:
-          GRN_PLUGIN_ERROR(ctx,
-                           GRN_OPERATION_NOT_SUPPORTED,
-                           "unsupported column type: %#x",
-                           column->header.type);
-          break;
+    GRN_TEXT_INIT(&delete_commands, 0);
+    cursor = grn_table_cursor_open(ctx, table, NULL, 0, NULL, 0, 0, -1,
+                                   GRN_CURSOR_BY_KEY);
+    while ((id = grn_table_cursor_next(ctx, cursor)) != GRN_ID_NIL) {
+      if (old_id != GRN_ID_NIL) { GRN_TEXT_PUTS(ctx, dumper->output, ",\n"); }
+      if (table->header.type == GRN_TABLE_NO_KEY && old_id + 1 < id) {
+        grn_id current_id;
+        for (current_id = old_id + 1; current_id < id; current_id++) {
+          GRN_TEXT_PUTS(ctx, dumper->output, "[],\n");
+          GRN_TEXT_PUTS(ctx, &delete_commands, "delete --table ");
+          dump_obj_name_raw(ctx, &delete_commands, table);
+          GRN_TEXT_PUTS(ctx, &delete_commands, " --id ");
+          grn_text_lltoa(ctx, &delete_commands, current_id);
+          GRN_TEXT_PUTC(ctx, &delete_commands, '\n');
         }
-        break;
-      case GRN_COLUMN_INDEX:
-        break;
-      case GRN_ACCESSOR:
-        {
-          GRN_OBJ_INIT(&buf, GRN_BULK, 0, range);
-          grn_obj_get_value(ctx, column, id, &buf);
-          /* XXX maybe, grn_obj_get_range() should not unconditionally return
-             GRN_DB_INT32 when column is GRN_ACCESSOR and
-             GRN_ACCESSOR_GET_VALUE */
-          if (is_value_column) {
-            buf.header.domain = grn_obj_get_range(ctx, table);
-          }
-          grn_text_otoj(ctx, dumper->output, &buf, NULL);
-          grn_obj_unlink(ctx, &buf);
-        }
-        break;
-      default:
-        GRN_PLUGIN_ERROR(ctx,
-                         GRN_OPERATION_NOT_SUPPORTED,
-                         "unsupported header type %#x",
-                         column->header.type);
-        break;
       }
+      dump_record(ctx, dumper, table, id, &columns, n_columns);
+
+      old_id = id;
     }
-    GRN_TEXT_PUTC(ctx, dumper->output, ']');
-    if (GRN_TEXT_LEN(dumper->output) >= DUMP_FLUSH_THRESHOLD_SIZE) {
-      grn_ctx_output_flush(ctx, 0);
-    }
+    grn_table_cursor_close(ctx, cursor);
+    GRN_TEXT_PUTS(ctx, dumper->output, "\n]\n");
+    GRN_TEXT_PUT(ctx, dumper->output,
+                 GRN_TEXT_VALUE(&delete_commands),
+                 GRN_TEXT_LEN(&delete_commands));
+    GRN_OBJ_FIN(ctx, &delete_commands);
   }
-  grn_table_cursor_close(ctx, cursor);
-  GRN_TEXT_PUTS(ctx, dumper->output, "\n]\n");
-  GRN_TEXT_PUT(ctx, dumper->output, GRN_TEXT_VALUE(&delete_commands),
-                            GRN_TEXT_LEN(&delete_commands));
-  GRN_OBJ_FIN(ctx, &delete_commands);
-
 exit :
-  GRN_OBJ_FIN(ctx, &column_name);
-
   for (i = 0; i < n_columns; i++) {
     grn_obj *column;
 
@@ -643,6 +704,21 @@ exit :
   }
 }
 
+static void
+dump_optionable_obj_string(grn_ctx *ctx,
+                           grn_dumper *dumper,
+                           grn_obj *string)
+{
+  const char *value = GRN_TEXT_VALUE(string);
+  size_t length = GRN_TEXT_LEN(string);
+
+  if (grn_proc_text_include_special_character(ctx, value, length)) {
+    grn_text_otoj(ctx, dumper->output, string, NULL);
+  } else {
+    GRN_TEXT_PUT(ctx, dumper->output, value, length);
+  }
+}
+
 static void
 dump_table(grn_ctx *ctx, grn_dumper *dumper, grn_obj *table)
 {
@@ -703,28 +779,28 @@ dump_table(grn_ctx *ctx, grn_dumper *dumper, grn_obj *table)
     grn_obj_unlink(ctx, range);
   }
   if (default_tokenizer) {
+    grn_obj sub_output;
     GRN_TEXT_PUTS(ctx, dumper->output, " --default_tokenizer ");
-    dump_obj_name(ctx, dumper, default_tokenizer);
+    GRN_TEXT_INIT(&sub_output, 0);
+    grn_table_get_default_tokenizer_string(ctx, table, &sub_output);
+    dump_optionable_obj_string(ctx, dumper, &sub_output);
+    GRN_OBJ_FIN(ctx, &sub_output);
   }
   if (normalizer) {
+    grn_obj sub_output;
     GRN_TEXT_PUTS(ctx, dumper->output, " --normalizer ");
-    dump_obj_name(ctx, dumper, normalizer);
+    GRN_TEXT_INIT(&sub_output, 0);
+    grn_table_get_normalizer_string(ctx, table, &sub_output);
+    dump_optionable_obj_string(ctx, dumper, &sub_output);
+    GRN_OBJ_FIN(ctx, &sub_output);
   }
-  if (table->header.type != GRN_TABLE_NO_KEY) {
-    int n_token_filters;
-
-    n_token_filters = GRN_BULK_VSIZE(token_filters) / sizeof(grn_obj *);
-    if (n_token_filters > 0) {
-      int i;
-      GRN_TEXT_PUTS(ctx, dumper->output, " --token_filters ");
-      for (i = 0; i < n_token_filters; i++) {
-        grn_obj *token_filter = GRN_PTR_VALUE_AT(token_filters, i);
-        if (i > 0) {
-          GRN_TEXT_PUTC(ctx, dumper->output, ',');
-        }
-        dump_obj_name(ctx, dumper, token_filter);
-      }
-    }
+  if (token_filters && GRN_BULK_VSIZE(token_filters) > 0) {
+    grn_obj sub_output;
+    GRN_TEXT_PUTS(ctx, dumper->output, " --token_filters ");
+    GRN_TEXT_INIT(&sub_output, 0);
+    grn_table_get_token_filters_string(ctx, table, &sub_output);
+    dump_optionable_obj_string(ctx, dumper, &sub_output);
+    GRN_OBJ_FIN(ctx, &sub_output);
   }
 
   GRN_TEXT_PUTC(ctx, dumper->output, '\n');
@@ -1014,6 +1090,11 @@ command_dump(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
   is_dump_configs = grn_plugin_proc_get_var_bool(ctx, user_data,
                                                  "dump_configs", -1,
                                                  GRN_TRUE);
+  dumper.is_sort_hash_table =
+    grn_plugin_proc_get_var_bool(ctx, user_data,
+                                 "sort_hash_table", -1,
+                                 GRN_FALSE);
+  GRN_TEXT_INIT(&(dumper.column_name_buffer), 0);
 
   grn_ctx_set_output_type(ctx, GRN_CONTENT_GROONGA_COMMAND_LIST);
 
@@ -1041,20 +1122,22 @@ command_dump(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
   if (is_dump_indexes) {
     dump_indexes(ctx, &dumper);
   }
-
   /* remove the last newline because another one will be added by the caller.
      maybe, the caller of proc functions currently doesn't consider the
      possibility of multiple-line output from proc functions. */
   if (GRN_BULK_VSIZE(dumper.output) > 0) {
     grn_bulk_truncate(ctx, dumper.output, GRN_BULK_VSIZE(dumper.output) - 1);
   }
+
+  GRN_OBJ_FIN(ctx, &(dumper.column_name_buffer));
+
   return NULL;
 }
 
 void
 grn_proc_init_dump(grn_ctx *ctx)
 {
-  grn_expr_var vars[6];
+  grn_expr_var vars[7];
 
   grn_plugin_expr_var_init(ctx, &(vars[0]), "tables", -1);
   grn_plugin_expr_var_init(ctx, &(vars[1]), "dump_plugins", -1);
@@ -1062,9 +1145,10 @@ grn_proc_init_dump(grn_ctx *ctx)
   grn_plugin_expr_var_init(ctx, &(vars[3]), "dump_records", -1);
   grn_plugin_expr_var_init(ctx, &(vars[4]), "dump_indexes", -1);
   grn_plugin_expr_var_init(ctx, &(vars[5]), "dump_configs", -1);
+  grn_plugin_expr_var_init(ctx, &(vars[6]), "sort_hash_table", -1);
   grn_plugin_command_create(ctx,
                             "dump", -1,
                             command_dump,
-                            6,
+                            sizeof(vars) / sizeof(vars[0]),
                             vars);
 }
