@@ -14,11 +14,11 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#include "groonga_in.h"
+#include "grn.h"
 #include <string.h>
 #include <stddef.h>
-#include "snip.h"
-#include "ctx.h"
+#include "grn_snip.h"
+#include "grn_ctx.h"
 
 #if !defined MAX
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
@@ -297,7 +297,7 @@ grn_snip_cond_reinit(snip_cond *cond)
   cond->stopflag = SNIPCOND_NONSTOP;
 }
 
-inline static char *
+grn_inline static char *
 grn_snip_strndup(grn_ctx *ctx, const char *string, unsigned int string_len)
 {
    char *copied_string;
@@ -306,12 +306,12 @@ grn_snip_strndup(grn_ctx *ctx, const char *string, unsigned int string_len)
    if (!copied_string) {
      return NULL;
    }
-   memcpy(copied_string, string, string_len);
+   grn_memcpy(copied_string, string, string_len);
    copied_string[string_len]= '\0'; /* not required, but for ql use */
    return copied_string;
 }
 
-inline static grn_rc
+grn_inline static grn_rc
 grn_snip_cond_set_tag(grn_ctx *ctx,
                       const char **dest_tag, size_t *dest_tag_len,
                       const char *tag, unsigned int tag_len,
@@ -434,7 +434,7 @@ grn_snip_find_firstbyte(const char *string, grn_encoding encoding, size_t offset
       offset += doffset;
     break;
   case GRN_ENC_UTF8:
-    while (string[offset] <= (char)0xc0)
+    while ((signed char)string[offset] <= (signed char)0xc0)
       offset += doffset;
     break;
   default:
@@ -443,7 +443,7 @@ grn_snip_find_firstbyte(const char *string, grn_encoding encoding, size_t offset
   return offset;
 }
 
-inline static grn_rc
+grn_inline static grn_rc
 grn_snip_set_default_tag(grn_ctx *ctx,
                          const char **dest_tag, size_t *dest_tag_len,
                          const char *tag, unsigned int tag_len,
@@ -775,7 +775,9 @@ grn_snip_get_result(grn_ctx *ctx, grn_obj *snip, const unsigned int index, char
       if (snip_->tag_result[j].end_offset > sres->end_offset) {
         continue;
       }
-      memcpy(p, snip_->tag_result[j].cond->opentag, snip_->tag_result[j].cond->opentag_len);
+      grn_memcpy(p,
+                 snip_->tag_result[j].cond->opentag,
+                 snip_->tag_result[j].cond->opentag_len);
       p += snip_->tag_result[j].cond->opentag_len;
     }
 
@@ -820,8 +822,9 @@ grn_snip_get_result(grn_ctx *ctx, grn_obj *snip, const unsigned int index, char
          snip_->tag_result[k].end_offset <= sres->end_offset; k--) {
       /* TODO: avoid all loop */
       if (snip_->tag_result[k].end_offset == i + 1) {
-        memcpy(p, snip_->tag_result[k].cond->closetag,
-               snip_->tag_result[k].cond->closetag_len);
+        grn_memcpy(p,
+                   snip_->tag_result[k].cond->closetag,
+                   snip_->tag_result[k].cond->closetag_len);
         p += snip_->tag_result[k].cond->closetag_len;
       }
       if (k <= sres->first_tag_result_idx) {
