@@ -724,3 +724,186 @@ grn_table_get_token_filters_string(grn_ctx *ctx,
                                       OPTION_NAME_TOKEN_FILTER,
                                       "token-filter");
 }
+
+static void
+grn_table_copy_same_key_type(grn_ctx *ctx, grn_obj *from, grn_obj *to)
+{
+  GRN_TABLE_EACH_BEGIN_FLAGS(ctx, from, cursor, from_id,
+                             GRN_CURSOR_BY_KEY | GRN_CURSOR_ASCENDING) {
+    void   *key;
+    int     key_size;
+    grn_id  to_id;
+
+    key_size = grn_table_cursor_get_key(ctx, cursor, &key);
+    to_id    = grn_table_add(ctx, to, key, key_size, NULL);
+    if (to_id == GRN_ID_NIL) {
+      char from_name[GRN_TABLE_MAX_KEY_SIZE];
+      int from_name_size;
+      char to_name[GRN_TABLE_MAX_KEY_SIZE];
+      int to_name_size;
+      grn_obj key_buffer;
+      grn_obj inspected_key;
+
+      from_name_size = grn_obj_name(ctx, from, from_name, sizeof(from_name));
+      to_name_size = grn_obj_name(ctx, to, to_name, sizeof(to_name));
+      if (from->header.domain == GRN_DB_SHORT_TEXT) {
+        GRN_SHORT_TEXT_INIT(&key_buffer, 0);
+      } else {
+        GRN_VALUE_FIX_SIZE_INIT(&key_buffer, 0, from->header.domain);
+      }
+      grn_bulk_write(ctx, &key_buffer, key, key_size);
+      GRN_TEXT_INIT(&inspected_key, 0);
+      grn_inspect(ctx, &inspected_key, &key_buffer);
+      ERR(GRN_INVALID_ARGUMENT,
+          "[table][copy] failed to copy key: <%.*s>: "
+          "<%.*s> -> <%.*s>",
+          (int)GRN_TEXT_LEN(&inspected_key),
+          GRN_TEXT_VALUE(&inspected_key),
+          from_name_size, from_name,
+          to_name_size, to_name);
+      GRN_OBJ_FIN(ctx, &inspected_key);
+      GRN_OBJ_FIN(ctx, &key_buffer);
+      break;
+    }
+  } GRN_TABLE_EACH_END(ctx, cursor);
+}
+
+static void
+grn_table_copy_different(grn_ctx *ctx, grn_obj *from, grn_obj *to)
+{
+  grn_obj from_key_buffer;
+  grn_obj to_key_buffer;
+
+  if (from->header.domain == GRN_DB_SHORT_TEXT) {
+    GRN_SHORT_TEXT_INIT(&from_key_buffer, 0);
+  } else {
+    GRN_VALUE_FIX_SIZE_INIT(&from_key_buffer, 0, from->header.domain);
+  }
+  if (to->header.domain == GRN_DB_SHORT_TEXT) {
+    GRN_SHORT_TEXT_INIT(&to_key_buffer, 0);
+  } else {
+    GRN_VALUE_FIX_SIZE_INIT(&to_key_buffer, 0, to->header.domain);
+  }
+
+  GRN_TABLE_EACH_BEGIN_FLAGS(ctx, from, cursor, from_id,
+                             GRN_CURSOR_BY_KEY | GRN_CURSOR_ASCENDING) {
+    void *key;
+    int key_size;
+    grn_rc cast_rc;
+    grn_id to_id;
+
+    GRN_BULK_REWIND(&from_key_buffer);
+    GRN_BULK_REWIND(&to_key_buffer);
+
+    key_size = grn_table_cursor_get_key(ctx, cursor, &key);
+    grn_bulk_write(ctx, &from_key_buffer, key, key_size);
+    cast_rc = grn_obj_cast(ctx, &from_key_buffer, &to_key_buffer, GRN_FALSE);
+    if (cast_rc != GRN_SUCCESS) {
+      char from_name[GRN_TABLE_MAX_KEY_SIZE];
+      int from_name_size;
+      char to_name[GRN_TABLE_MAX_KEY_SIZE];
+      int to_name_size;
+      grn_obj *to_key_type;
+      grn_obj inspected_key;
+      grn_obj inspected_to_key_type;
+
+      from_name_size = grn_obj_name(ctx, from, from_name, sizeof(from_name));
+      to_name_size = grn_obj_name(ctx, to, to_name, sizeof(to_name));
+      to_key_type = grn_ctx_at(ctx, to->header.domain);
+      GRN_TEXT_INIT(&inspected_key, 0);
+      GRN_TEXT_INIT(&inspected_to_key_type, 0);
+      grn_inspect(ctx, &inspected_key, &from_key_buffer);
+      grn_inspect(ctx, &inspected_to_key_type, to_key_type);
+      ERR(cast_rc,
+          "[table][copy] failed to cast key: <%.*s> -> %.*s: "
+          "<%.*s> -> <%.*s>",
+          (int)GRN_TEXT_LEN(&inspected_key),
+          GRN_TEXT_VALUE(&inspected_key),
+          (int)GRN_TEXT_LEN(&inspected_to_key_type),
+          GRN_TEXT_VALUE(&inspected_to_key_type),
+          from_name_size, from_name,
+          to_name_size, to_name);
+      GRN_OBJ_FIN(ctx, &inspected_key);
+      GRN_OBJ_FIN(ctx, &inspected_to_key_type);
+      break;
+    }
+
+    to_id = grn_table_add(ctx, to,
+                          GRN_BULK_HEAD(&to_key_buffer),
+                          GRN_BULK_VSIZE(&to_key_buffer),
+                          NULL);
+    if (to_id == GRN_ID_NIL) {
+      char from_name[GRN_TABLE_MAX_KEY_SIZE];
+      int from_name_size;
+      char to_name[GRN_TABLE_MAX_KEY_SIZE];
+      int to_name_size;
+      grn_obj inspected_from_key;
+      grn_obj inspected_to_key;
+
+      from_name_size = grn_obj_name(ctx, from, from_name, sizeof(from_name));
+      to_name_size = grn_obj_name(ctx, to, to_name, sizeof(to_name));
+      GRN_TEXT_INIT(&inspected_from_key, 0);
+      GRN_TEXT_INIT(&inspected_to_key, 0);
+      grn_inspect(ctx, &inspected_from_key, &from_key_buffer);
+      grn_inspect(ctx, &inspected_to_key, &to_key_buffer);
+      ERR(GRN_INVALID_ARGUMENT,
+          "[table][copy] failed to copy key: <%.*s> -> <%.*s>: "
+          "<%.*s> -> <%.*s>",
+          (int)GRN_TEXT_LEN(&inspected_from_key),
+          GRN_TEXT_VALUE(&inspected_from_key),
+          (int)GRN_TEXT_LEN(&inspected_to_key),
+          GRN_TEXT_VALUE(&inspected_to_key),
+          from_name_size, from_name,
+          to_name_size, to_name);
+      GRN_OBJ_FIN(ctx, &inspected_from_key);
+      GRN_OBJ_FIN(ctx, &inspected_to_key);
+      break;
+    }
+  } GRN_TABLE_EACH_END(ctx, cursor);
+  GRN_OBJ_FIN(ctx, &from_key_buffer);
+  GRN_OBJ_FIN(ctx, &to_key_buffer);
+}
+
+grn_rc
+grn_table_copy(grn_ctx *ctx, grn_obj *from, grn_obj *to)
+{
+  GRN_API_ENTER;
+
+  if (from->header.type == GRN_TABLE_NO_KEY ||
+      to->header.type == GRN_TABLE_NO_KEY) {
+    char from_name[GRN_TABLE_MAX_KEY_SIZE];
+    int from_name_size;
+    char to_name[GRN_TABLE_MAX_KEY_SIZE];
+    int to_name_size;
+
+    from_name_size = grn_obj_name(ctx, from, from_name, sizeof(from_name));
+    to_name_size = grn_obj_name(ctx, to, to_name, sizeof(to_name));
+    ERR(GRN_OPERATION_NOT_SUPPORTED,
+        "[table][copy] copy from/to TABLE_NO_KEY isn't supported: "
+        "<%.*s> -> <%.*s>",
+        from_name_size, from_name,
+        to_name_size, to_name);
+    goto exit;
+  }
+
+  if (from == to) {
+    char from_name[GRN_TABLE_MAX_KEY_SIZE];
+    int from_name_size;
+
+    from_name_size = grn_obj_name(ctx, from, from_name, sizeof(from_name));
+    ERR(GRN_OPERATION_NOT_SUPPORTED,
+        "[table][copy] from table and to table is the same: "
+        "<%.*s>",
+        from_name_size, from_name);
+    goto exit;
+  }
+
+  if (from->header.domain == to->header.domain) {
+    grn_table_copy_same_key_type(ctx, from, to);
+  } else {
+    grn_table_copy_different(ctx, from, to);
+  }
+
+exit :
+  GRN_API_RETURN(ctx->rc);
+}
