@@ -153,6 +153,13 @@ grn_token_cursor_open(grn_ctx *ctx, grn_obj *table,
                               &(token_cursor->curr_size),
                               NULL);
     token_cursor->curr = (const unsigned char *)normalized;
+    grn_token_set_data(ctx,
+                       &(token_cursor->tokenizer.current_token),
+                       token_cursor->curr,
+                       token_cursor->curr_size);
+    grn_token_set_status(ctx,
+                         &(token_cursor->tokenizer.current_token),
+                         GRN_TOKEN_LAST);
   }
 
   if (ctx->rc == GRN_SUCCESS) {
@@ -311,6 +318,7 @@ grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor)
         }
       }
     } else {
+      grn_token_cursor_next_apply_token_filters(ctx, token_cursor);
       token_cursor->status = GRN_TOKEN_CURSOR_DONE;
     }
     if (token_cursor->mode == GRN_TOKENIZE_ADD) {
