@@ -1,6 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2012-2018 Brazil
+  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -167,6 +168,7 @@ grn_tokenizer_query_init(grn_ctx *ctx, grn_tokenizer_query *query)
   query->token_mode = query->tokenize_mode;
   query->lexicon = NULL;
   query->encoding = ctx->encoding;
+  query->token_filter_index = 0;
 
   query->need_normalize = GRN_TRUE;
   query->need_delimiter_check = GRN_TRUE;
@@ -427,6 +429,24 @@ grn_tokenizer_query_get_lexicon(grn_ctx *ctx, grn_tokenizer_query *query)
   GRN_API_RETURN(query->lexicon);
 }
 
+grn_rc
+grn_tokenizer_query_set_token_filter_index(grn_ctx *ctx,
+                                           grn_tokenizer_query *query,
+                                           unsigned int index)
+{
+  GRN_API_ENTER;
+  query->token_filter_index = index;
+  GRN_API_RETURN(ctx->rc);
+}
+
+unsigned int
+grn_tokenizer_query_get_token_filter_index(grn_ctx *ctx,
+                                           grn_tokenizer_query *query)
+{
+  GRN_API_ENTER;
+  GRN_API_RETURN(query->token_filter_index);
+}
+
 void
 grn_tokenizer_token_init(grn_ctx *ctx, grn_tokenizer_token *token)
 {
@@ -490,6 +510,41 @@ grn_tokenizer_tokenized_delimiter_next(grn_ctx *ctx,
   return next_start;
 }
 
+const char *
+grn_tokenizer_next_by_tokenized_delimiter(grn_ctx *ctx,
+                                          grn_token *token,
+                                          const char *str_ptr,
+                                          unsigned int str_length,
+                                          grn_encoding encoding)
+{
+  size_t char_length = 0;
+  const char *start = str_ptr;
+  const char *current;
+  const char *end = str_ptr + str_length;
+  const char *next_start = NULL;
+
+  for (current = start; current < end; current += char_length) {
+    char_length = grn_charlen_(ctx, current, end, encoding);
+    if (char_length == 0) {
+      break;
+    }
+    if (grn_tokenizer_is_tokenized_delimiter(ctx, current, char_length,
+                                             encoding)) {
+      next_start = str_ptr + (current - start + char_length);
+      break;
+    }
+  }
+
+  grn_token_set_data(ctx, token, start, current - start);
+  if (current == end) {
+    grn_token_set_status(ctx, token, GRN_TOKEN_LAST);
+  } else {
+    grn_token_set_status(ctx, token, GRN_TOKEN_CONTINUE);
+  }
+
+  return next_start;
+}
+
 grn_rc
 grn_tokenizer_register(grn_ctx *ctx, const char *plugin_name_ptr,
                        unsigned int plugin_name_length,
@@ -540,9 +595,13 @@ grn_tokenizer_create(grn_ctx *ctx,
                               0,
                               NULL);
   if (!tokenizer) {
+    if (name_length < 0) {
+      name_length = strlen(name);
+    }
     GRN_PLUGIN_ERROR(ctx,
                      GRN_TOKENIZER_ERROR,
-                     "[tokenizer][create] failed to create");
+                     "[tokenizer][create] failed to create: <%.*s>",
+                     name_length, name);
   }
 
   GRN_API_RETURN(tokenizer);
