@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2010-2017 Brazil
+  Copyright(C) 2010-2018 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -502,114 +502,8 @@ grn_vector_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *vector)
 static grn_rc
 grn_accessor_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
 {
-  grn_accessor *accessor = (grn_accessor *)obj;
-
   GRN_TEXT_PUTS(ctx, buf, "#<accessor ");
-  for (; accessor; accessor = accessor->next) {
-    grn_bool show_obj_name = GRN_FALSE;
-    grn_bool show_obj_domain_name = GRN_FALSE;
-
-    if (accessor != (grn_accessor *)obj) {
-      GRN_TEXT_PUTS(ctx, buf, ".");
-    }
-    switch (accessor->action) {
-    case GRN_ACCESSOR_GET_ID :
-      GRN_TEXT_PUT(ctx,
-                   buf,
-                   GRN_COLUMN_NAME_ID,
-                   GRN_COLUMN_NAME_ID_LEN);
-      show_obj_name = GRN_TRUE;
-      break;
-    case GRN_ACCESSOR_GET_KEY :
-      GRN_TEXT_PUT(ctx,
-                   buf,
-                   GRN_COLUMN_NAME_KEY,
-                   GRN_COLUMN_NAME_KEY_LEN);
-      show_obj_name = GRN_TRUE;
-      break;
-    case GRN_ACCESSOR_GET_VALUE :
-      GRN_TEXT_PUT(ctx,
-                   buf,
-                   GRN_COLUMN_NAME_VALUE,
-                   GRN_COLUMN_NAME_VALUE_LEN);
-      show_obj_name = GRN_TRUE;
-      break;
-    case GRN_ACCESSOR_GET_SCORE :
-      GRN_TEXT_PUT(ctx,
-                   buf,
-                   GRN_COLUMN_NAME_SCORE,
-                   GRN_COLUMN_NAME_SCORE_LEN);
-      break;
-    case GRN_ACCESSOR_GET_NSUBRECS :
-      GRN_TEXT_PUT(ctx,
-                   buf,
-                   GRN_COLUMN_NAME_NSUBRECS,
-                   GRN_COLUMN_NAME_NSUBRECS_LEN);
-      break;
-    case GRN_ACCESSOR_GET_MAX :
-      GRN_TEXT_PUT(ctx,
-                   buf,
-                   GRN_COLUMN_NAME_MAX,
-                   GRN_COLUMN_NAME_MAX_LEN);
-      break;
-    case GRN_ACCESSOR_GET_MIN :
-      GRN_TEXT_PUT(ctx,
-                   buf,
-                   GRN_COLUMN_NAME_MIN,
-                   GRN_COLUMN_NAME_MIN_LEN);
-      break;
-    case GRN_ACCESSOR_GET_SUM :
-      GRN_TEXT_PUT(ctx,
-                   buf,
-                   GRN_COLUMN_NAME_SUM,
-                   GRN_COLUMN_NAME_SUM_LEN);
-      break;
-    case GRN_ACCESSOR_GET_AVG :
-      GRN_TEXT_PUT(ctx,
-                   buf,
-                   GRN_COLUMN_NAME_AVG,
-                   GRN_COLUMN_NAME_AVG_LEN);
-      break;
-    case GRN_ACCESSOR_GET_COLUMN_VALUE :
-      grn_column_name_(ctx, accessor->obj, buf);
-      show_obj_domain_name = GRN_TRUE;
-      break;
-    case GRN_ACCESSOR_GET_DB_OBJ :
-      grn_text_printf(ctx, buf, "(_db)");
-      break;
-    case GRN_ACCESSOR_LOOKUP :
-      grn_text_printf(ctx, buf, "(_lookup)");
-      break;
-    case GRN_ACCESSOR_FUNCALL :
-      grn_text_printf(ctx, buf, "(_funcall)");
-      break;
-    default :
-      grn_text_printf(ctx, buf, "(unknown:%u)", accessor->action);
-      break;
-    }
-
-    if (show_obj_name || show_obj_domain_name) {
-      grn_obj *target = accessor->obj;
-      char name[GRN_TABLE_MAX_KEY_SIZE];
-      int name_size;
-
-      if (show_obj_domain_name) {
-        target = grn_ctx_at(ctx, target->header.domain);
-      }
-
-      name_size = grn_obj_name(ctx,
-                               target,
-                               name,
-                               GRN_TABLE_MAX_KEY_SIZE);
-      GRN_TEXT_PUTS(ctx, buf, "(");
-      if (name_size == 0) {
-        GRN_TEXT_PUTS(ctx, buf, "anonymous");
-      } else {
-        GRN_TEXT_PUT(ctx, buf, name, name_size);
-      }
-      GRN_TEXT_PUTS(ctx, buf, ")");
-    }
-  }
+  grn_accessor_name(ctx, obj, buf);
   GRN_TEXT_PUTS(ctx, buf, ">");
 
   return GRN_SUCCESS;
@@ -943,12 +837,20 @@ grn_table_normalizer_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
 static grn_rc
 grn_table_keys_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
 {
+  grn_obj *score = NULL;
   grn_table_cursor *tc;
   int max_n_keys = 10;
 
   /* TODO */
   /* max_n_keys = grn_atoi(grn_getenv("GRN_INSPECT_TABLE_MAX_N_KEYS")); */
 
+  if (obj->header.flags & GRN_OBJ_WITH_SUBREC) {
+    score = grn_obj_column(ctx,
+                           obj,
+                           GRN_COLUMN_NAME_SCORE,
+                           GRN_COLUMN_NAME_SCORE_LEN);
+  }
+
   GRN_TEXT_PUTS(ctx, buf, "keys:[");
   tc = grn_table_cursor_open(ctx, obj, NULL, 0, NULL, 0,
                              0, -1, GRN_CURSOR_ASCENDING);
@@ -956,7 +858,9 @@ grn_table_keys_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
     int i = 0;
     grn_id id;
     grn_obj key;
+    grn_obj score_value;
     GRN_OBJ_INIT(&key, GRN_BULK, 0, obj->header.domain);
+    GRN_FLOAT_INIT(&score_value, 0);
     while ((id = grn_table_cursor_next(ctx, tc))) {
       if (max_n_keys > 0 && i >= max_n_keys) {
         GRN_TEXT_PUTS(ctx, buf, ", ...");
@@ -966,12 +870,24 @@ grn_table_keys_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
       grn_table_get_key2(ctx, obj, id, &key);
       grn_inspect(ctx, buf, &key);
       GRN_BULK_REWIND(&key);
+      if (score) {
+        GRN_BULK_REWIND(&score_value);
+        grn_obj_get_value(ctx, score, id, &score_value);
+        GRN_TEXT_PUTS(ctx, buf, "(");
+        grn_inspect(ctx, buf, &score_value);
+        GRN_TEXT_PUTS(ctx, buf, ")");
+      }
     }
     GRN_OBJ_FIN(ctx, &key);
+    GRN_OBJ_FIN(ctx, &score_value);
     grn_table_cursor_close(ctx, tc);
   }
   GRN_TEXT_PUTS(ctx, buf, "]");
 
+  if (score) {
+    grn_obj_close(ctx, score);
+  }
+
   return GRN_SUCCESS;
 }
 
@@ -1222,10 +1138,12 @@ grn_json_load_open_brace_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
 }
 
 static grn_rc
-grn_record_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
+grn_record_inspect_common(grn_ctx *ctx,
+                          grn_obj *buf,
+                          grn_obj *obj,
+                          grn_bool with_columns)
 {
   grn_obj *table;
-  grn_hash *cols;
 
   table = grn_ctx_at(ctx, obj->header.domain);
   GRN_TEXT_PUTS(ctx, buf, "#<record:");
@@ -1245,41 +1163,59 @@ grn_record_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
   } else {
     grn_id id;
 
-  id = GRN_RECORD_VALUE(obj);
-  grn_text_lltoa(ctx, buf, id);
+    id = GRN_RECORD_VALUE(obj);
+    grn_text_lltoa(ctx, buf, id);
 
-  if (table && grn_table_at(ctx, table, id)) {
-    if (table->header.type != GRN_TABLE_NO_KEY) {
-      grn_obj key;
-      GRN_TEXT_PUTS(ctx, buf, " key:");
-      GRN_OBJ_INIT(&key, GRN_BULK, 0, table->header.domain);
-      grn_table_get_key2(ctx, table, id, &key);
-      grn_inspect(ctx, buf, &key);
-      GRN_OBJ_FIN(ctx, &key);
-    }
-    if ((cols = grn_hash_create(ctx, NULL, sizeof(grn_id), 0,
-                                GRN_OBJ_TABLE_HASH_KEY|GRN_HASH_TINY))) {
-      if (grn_table_columns(ctx, table, "", 0, (grn_obj *)cols)) {
-        grn_id *key;
-        GRN_HASH_EACH(ctx, cols, column_id, &key, NULL, NULL, {
-            grn_obj *col = grn_ctx_at(ctx, *key);
-            if (col) {
-              grn_obj value;
-              GRN_TEXT_INIT(&value, 0);
-              GRN_TEXT_PUTS(ctx, buf, " ");
-              grn_column_name_(ctx, col, buf);
-              GRN_TEXT_PUTS(ctx, buf, ":");
-              grn_obj_get_value(ctx, col, id, &value);
-              grn_inspect(ctx, buf, &value);
-              GRN_OBJ_FIN(ctx, &value);
-            }
-          });
+    if (table && grn_table_at(ctx, table, id)) {
+      if (table->header.type != GRN_TABLE_NO_KEY) {
+        grn_obj key;
+        GRN_TEXT_PUTS(ctx, buf, " key:");
+        GRN_OBJ_INIT(&key, GRN_BULK, 0, table->header.domain);
+        grn_table_get_key2(ctx, table, id, &key);
+        grn_inspect(ctx, buf, &key);
+        GRN_OBJ_FIN(ctx, &key);
       }
-      grn_hash_close(ctx, cols);
+
+      if (with_columns) {
+        grn_hash *columns;
+
+        columns = grn_hash_create(ctx,
+                                  NULL,
+                                  sizeof(grn_id),
+                                  0,
+                                  GRN_OBJ_TABLE_HASH_KEY|GRN_HASH_TINY);
+        if (columns) {
+          if (grn_table_columns(ctx, table, "", 0, (grn_obj *)columns)) {
+            grn_obj column_value;
+
+            GRN_VOID_INIT(&column_value);
+            GRN_HASH_EACH_BEGIN(ctx, columns, cursor, id) {
+              void *key;
+              grn_id column_id;
+              grn_obj *column;
+
+              grn_hash_cursor_get_key(ctx, cursor, &key);
+              column_id = *((grn_id *)key);
+              column = grn_ctx_at(ctx, column_id);
+              if (!column) {
+                continue;
+              }
+
+              GRN_TEXT_PUTS(ctx, buf, " ");
+              grn_column_name_(ctx, column, buf);
+              GRN_TEXT_PUTS(ctx, buf, ":");
+              GRN_BULK_REWIND(&column_value);
+              grn_obj_get_value(ctx, column, id, &column_value);
+              grn_inspect(ctx, buf, &column_value);
+            } GRN_HASH_EACH_END(ctx, cursor);
+            GRN_OBJ_FIN(ctx, &column_value);
+          }
+          grn_hash_close(ctx, columns);
+        }
+      }
+    } else {
+      GRN_TEXT_PUTS(ctx, buf, "(nonexistent)");
     }
-  } else {
-    GRN_TEXT_PUTS(ctx, buf, "(nonexistent)");
-  }
   }
 
   GRN_TEXT_PUTS(ctx, buf, ">");
@@ -1287,8 +1223,23 @@ grn_record_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
   return GRN_SUCCESS;
 }
 
+grn_rc
+grn_record_inspect_without_columns(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
+{
+  return grn_record_inspect_common(ctx, buf, obj, GRN_FALSE);
+}
+
 static grn_rc
-grn_uvector_record_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
+grn_record_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
+{
+  return grn_record_inspect_common(ctx, buf, obj, GRN_TRUE);
+}
+
+static grn_rc
+grn_uvector_record_inspect_common(grn_ctx *ctx,
+                                  grn_obj *buf,
+                                  grn_obj *obj,
+                                  grn_bool with_columns)
 {
   unsigned int i, n = 0;
   grn_obj record;
@@ -1307,7 +1258,7 @@ grn_uvector_record_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
     id = grn_uvector_get_element(ctx, obj, i, &weight);
     GRN_TEXT_PUTS(ctx, buf, "#<element record:");
     GRN_RECORD_SET(ctx, &record, id);
-    grn_inspect(ctx, buf, &record);
+    grn_record_inspect_common(ctx, buf, &record, with_columns);
     grn_text_printf(ctx, buf, ", weight:%u>", weight);
   }
   GRN_TEXT_PUTS(ctx, buf, "]");
@@ -1316,11 +1267,25 @@ grn_uvector_record_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
   return GRN_SUCCESS;
 }
 
+grn_rc
+grn_uvector_record_inspect_without_columns(grn_ctx *ctx,
+                                           grn_obj *buf,
+                                           grn_obj *obj)
+{
+  return grn_uvector_record_inspect_common(ctx, buf, obj, GRN_FALSE);
+}
+
+static grn_rc
+grn_uvector_record_inspect(grn_ctx *ctx,
+                           grn_obj *buf,
+                           grn_obj *obj)
+{
+  return grn_uvector_record_inspect_common(ctx, buf, obj, GRN_TRUE);
+}
+
 grn_obj *
 grn_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
 {
-  grn_obj *domain;
-
   if (!buffer) {
     buffer = grn_obj_open(ctx, GRN_BULK, 0, GRN_DB_TEXT);
   }
@@ -1350,37 +1315,20 @@ grn_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
       grn_json_load_open_brace_inspect(ctx, buffer, obj);
       return buffer;
     default :
-      domain = grn_ctx_at(ctx, obj->header.domain);
-      if (domain) {
-        grn_id type = domain->header.type;
-        switch (type) {
-        case GRN_TABLE_HASH_KEY :
-        case GRN_TABLE_PAT_KEY :
-        case GRN_TABLE_NO_KEY :
-          grn_record_inspect(ctx, buffer, obj);
-          return buffer;
-        default :
-          break;
-        }
+      if (grn_obj_is_table(ctx, grn_ctx_at(ctx, obj->header.domain))) {
+        grn_record_inspect(ctx, buffer, obj);
+        return buffer;
       }
+      break;
     }
     break;
   case GRN_PTR :
     grn_ptr_inspect(ctx, buffer, obj);
     break;
   case GRN_UVECTOR :
-    domain = grn_ctx_at(ctx, obj->header.domain);
-    if (domain) {
-      grn_id type = domain->header.type;
-      switch (type) {
-      case GRN_TABLE_HASH_KEY :
-      case GRN_TABLE_PAT_KEY :
-      case GRN_TABLE_NO_KEY :
-        grn_uvector_record_inspect(ctx, buffer, obj);
-        return buffer;
-      default :
-        break;
-      }
+    if (grn_obj_is_table(ctx, grn_ctx_at(ctx, obj->header.domain))) {
+      grn_uvector_record_inspect(ctx, buffer, obj);
+      return buffer;
     }
     break;
   case GRN_PVECTOR :
