@@ -28,6 +28,9 @@ module Groonga
                  "match_columns",
                  "query",
                  "drilldown_filter",
+                 "load_table",
+                 "load_columns",
+                 "load_values",
                ])
 
       def run_body(input)
@@ -87,9 +90,14 @@ module Groonga
         key << "#{input[:drilldown_calc_types]}\0"
         key << "#{input[:drilldown_calc_target]}\0"
         key << "#{input[:drilldown_filter]}\0"
+        key << "#{input[:post_filter]}\0"
+        key << "#{input[:load_table]}\0"
+        key << "#{input[:load_columns]}\0"
+        key << "#{input[:load_values]}\0"
         labeled_drilldowns = LabeledDrilldowns.parse(input).sort_by(&:label)
         labeled_drilldowns.each do |drilldown|
           key << "#{drilldown.label}\0"
+          key << "#{drilldown.table}\0"
           key << "#{drilldown.keys.join(',')}\0"
           key << "#{drilldown.output_columns}\0"
           key << "#{drilldown.offset}\0"
@@ -271,9 +279,12 @@ module Groonga
         attr_reader :limit
         attr_reader :sort_keys
         attr_reader :output_columns
+        attr_reader :post_filter
+        attr_reader :load_table
+        attr_reader :load_columns
+        attr_reader :load_values
         attr_reader :dynamic_columns
         attr_reader :result_sets
-        attr_reader :unsorted_result_sets
         attr_reader :plain_drilldown
         attr_reader :labeled_drilldowns
         attr_reader :temporary_tables
@@ -288,12 +299,14 @@ module Groonga
           @limit = (@input[:limit] || 10).to_i
           @sort_keys = parse_keys(@input[:sort_keys] || @input[:sortby])
           @output_columns = @input[:output_columns] || "_id, _key, *"
+          @post_filter = @input[:post_filter]
+          @load_table = @input[:load_table]
+          @load_columns = @input[:load_columns]
+          @load_values = @input[:load_values]
 
           @dynamic_columns = DynamicColumns.parse(@input)
 
           @result_sets = []
-          @unsorted_result_sets = []
-
           @plain_drilldown = PlainDrilldownExecuteContext.new(@input)
           @labeled_drilldowns = LabeledDrilldowns.parse(@input)
 
@@ -303,13 +316,6 @@ module Groonga
         end
 
         def close
-          @result_sets.each do |result_set|
-            result_set.close if result_set.temporary?
-          end
-          @unsorted_result_sets.each do |result_set|
-            result_set.close if result_set.temporary?
-          end
-
           @plain_drilldown.close
           @labeled_drilldowns.close
 
@@ -536,6 +542,7 @@ module Groonga
 
         def execute
           execute_search
+          execute_load_table
           if @context.plain_drilldown.have_keys?
             execute_plain_drilldown
           elsif @context.labeled_drilldowns.have_keys?
@@ -562,6 +569,7 @@ module Groonga
           if @context.result_sets.empty?
             result_set = HashTable.create(:flags => ObjectFlags::WITH_SUBREC,
                                           :key_type => first_shard.table)
+            @context.temporary_tables << result_set
             @context.dynamic_columns.each_initial do |dynamic_column|
               dynamic_column.apply(result_set)
             end
@@ -572,6 +580,35 @@ module Groonga
           end
         end
 
+        def execute_load_table
+          load_table = @context.load_table
+          load_columns = @context.load_columns
+          load_values = @context.load_values
+          return if load_table.nil?
+          return if load_columns.nil?
+          return if load_values.nil?
+
+          to = Context.instance[load_table]
+          to_columns = []
+          begin
+            load_columns.split(/\s*,\s*/).each do |name|
+              to_columns << to.find_column(name)
+            end
+            @context.result_sets.each do |result_set|
+              output_columns = result_set.parse_output_columns(load_values)
+              begin
+                output_columns.apply(to_columns)
+              ensure
+                output_columns.close
+              end
+            end
+          ensure
+            to_columns.each do |column|
+              column.close if column.is_a?(Accessor)
+            end
+          end
+        end
+
         def execute_plain_drilldown
           drilldown = @context.plain_drilldown
           group_result = TableGroupResult.new
@@ -682,9 +719,10 @@ module Groonga
           @match_columns = @context.match_columns
           @query = @context.query
           @filter = @context.filter
+          @post_filter = @context.post_filter
           @sort_keys = @context.sort_keys
           @result_sets = @context.result_sets
-          @unsorted_result_sets = @context.unsorted_result_sets
+          @temporary_tables = @context.temporary_tables
 
           @target_range = @context.enumerator.target_range
 
@@ -714,7 +752,7 @@ module Groonga
                 @target_table = @target_table.select(expression)
                 @cover_type = :all
               end
-              @context.temporary_tables << @target_table
+              @temporary_tables << @target_table
             end
             dynamic_column.apply(@target_table)
           end
@@ -742,8 +780,8 @@ module Groonga
         private
         def filter_shard_all(expression_builder)
           if @query.nil? and @filter.nil?
+            @temporary_tables.delete(@target_table)
             add_result_set(@target_table, nil)
-            @context.temporary_tables.delete(@target_table)
           else
             filter_table do |expression|
               expression_builder.build_all(expression)
@@ -778,6 +816,12 @@ module Groonga
           add_result_set(table.select(expression), expression)
         end
 
+        def apply_post_filter(table)
+          expression = create_expression(table)
+          expression.parse(@post_filter)
+          table.select(expression)
+        end
+
         def add_result_set(result_set, condition)
           query_logger.log(:size, ":",
                            "select(#{result_set.size})[#{@shard.table_name}]")
@@ -787,18 +831,29 @@ module Groonga
             return
           end
 
-          @context.dynamic_columns.each_filtered do |dynamic_column|
-            if result_set == @shard.table
+          if result_set == @shard.table
+            if @context.dynamic_columns.have_filtered?
               result_set = result_set.select_all
+              @temporary_tables << result_set
             end
+          else
+            @temporary_tables << result_set
+          end
+
+          @context.dynamic_columns.each_filtered do |dynamic_column|
             dynamic_column.apply(result_set, condition)
           end
 
+          if @post_filter
+            result_set = apply_post_filter(result_set)
+            @temporary_tables << result_set
+          end
+
           if @sort_keys.empty?
             @result_sets << result_set
           else
-            @unsorted_result_sets << result_set
             sorted_result_set = result_set.sort(@sort_keys)
+            @temporary_tables << sorted_result_set
             @result_sets << sorted_result_set
           end
         end
