@@ -15,7 +15,7 @@ build_top_dir=$(cd -P "$build_top_dir" 2>/dev/null || cd "$build_top_dir"; pwd)
 n_processors=1
 case `uname` in
   Linux)
-    n_processors="$(grep '^processor' /proc/cpuinfo | wc -l)"
+    n_processors="$(nproc)"
     ;;
   Darwin)
     n_processors="$(/usr/sbin/sysctl -n hw.ncpu)"
@@ -35,8 +35,10 @@ fi
 
 . "${build_top_dir}/config.sh"
 
-GRN_PLUGINS_DIR="$build_top_dir/plugins"
+GRN_PLUGINS_DIR="$source_top_dir/plugins"
+GRN_PLUGINS_PATH="$build_top_dir/plugins"
 export GRN_PLUGINS_DIR
+export GRN_PLUGINS_PATH
 
 GRN_RUBY_SCRIPTS_DIR="$source_top_dir/lib/mrb/scripts"
 export GRN_RUBY_SCRIPTS_DIR
