@@ -61,22 +61,6 @@ static grn_ctx context;
 static grn_obj *database;
 static grn_obj buffer;
 
-static const gchar text_ja_utf8[] =
-  "Groongaは組み込み型の全文検索エンジンです。DBMSやスクリプト言語処理系等に\n"
-  "組み込むことによって、その全文検索機能を強化することができます。n-gram\n"
-  "インデックスと単語インデックスの特徴を兼ね備えた、高速かつ高精度な転置\n"
-  "インデックスタイプのエンジンです。コンパクトな実装ですが、大規模な文書\n"
-  "量と検索要求を処理できるように設計されています。また、純粋なn-gramイン\n"
-  "デックスの作成も可能です。";
-
-static const gchar normalized_text_ja_utf8[] =
-  "groongaは組み込み型の全文検索エンジンです。dbmsやスクリプト言語処理系等に"
-  "組み込むことによって、その全文検索機能を強化することができます。n-gram"
-  "インデックスと単語インデックスの特徴を兼ね備えた、高速かつ高精度な転置"
-  "インデックスタイプのエンジンです。コンパクトな実装ですが、大規模な文書"
-  "量と検索要求を処理できるように設計されています。また、純粋なn-gramイン"
-  "デックスの作成も可能です。";
-
 void
 setup (void)
 {
